<?php
declare(strict_types = 1);

/**
 * Manipulation d'images avec GD.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class GD
{
	/**
	 * Crée une image GD du fichier $file.
	 *
	 * @param string $file
	 *   Chemin du fichier.
	 *
	 * @return mixed
	 */
	public static function createImage(string $file)
	{
		switch (Image::getTypeSize($file)['filetype'])
		{
			case IMAGETYPE_GIF :
				return imagecreatefromgif($file);

			case IMAGETYPE_JPEG :
				return imagecreatefromjpeg($file);

			case IMAGETYPE_PNG :
				return imagecreatefrompng($file);

			case IMAGETYPE_WEBP :
				return imagecreatefromwebp($file);
		}

		return FALSE;
	}

	/**
	 * Enregistre une image GD dans un fichier.
	 *
	 * @param resource $gd_image
	 *   Identifiant de l'image créée par GD.
	 * @param string $file
	 *   Chemin du fichier.
	 * @param int $type
	 *   Type du fichier.
	 * @param int $quality
	 *   Qualité de l'image enregistrée.
	 *
	 * @return bool
	 */
	public static function file(&$gd_image, string $file, int $type, int $quality): bool
	{
		File::makeWritable(dirname($file));

		if ($quality > 100)
		{
			$quality = 100;
		}

		switch ($type)
		{
			case IMAGETYPE_GIF :
				if (!imagegif($gd_image, $file))
				{
					return FALSE;
				}
				break;

			case IMAGETYPE_JPEG :
				if (!imagejpeg($gd_image, $file, $quality))
				{
					return FALSE;
				}
				break;

			case IMAGETYPE_PNG :
				if (!imagepng($gd_image, $file, (int) round($quality / 11)))
				{
					return FALSE;
				}
				break;

			case IMAGETYPE_WEBP :
				if (!imagewebp($gd_image, $file, $quality))
				{
					return FALSE;
				}
				break;
		}

		return imagedestroy($gd_image);
	}

	/**
	 * Découpe et redimensionne une image.
	 *
	 * @param resource $src_img
	 *   Ressource de l'image source.
	 * @param int $src_x
	 *   Coordonnée X de l'image source.
	 * @param int $src_y
	 *   Coordonnée Y de l'image source.
	 * @param int $src_w
	 *   Largeur de l'image source.
	 * @param int $src_h
	 *   Hauteur de l'image source.
	 * @param int $dst_x
	 *   Coordonnée X de l'image destination.
	 * @param int $dst_y
	 *   Coordonnée Y de l'image destination.
	 * @param int $dst_w
	 *   Largeur de l'image destination.
	 * @param int $dst_h
	 *   Hauteur de l'image destination.
	 * @param int $dst_canvas_w
	 *   Largeur de l'image finale.
	 * @param int $dst_canvas_h
	 *   Hauteur de l'image finale.
	 * @param int $bg_color_red
	 *   Valeur pour le composant rouge de la couleur de fond.
	 * @param int $bg_color_green
	 *   Valeur pour le composant vert de la couleur de fond.
	 * @param int $bg_color_blue
	 *   Valeur pour le composant bleu de la couleur de fond.
	 *
	 * @return resource
	 *   Ressource de l'image destination.
	 */
	public static function resize($src_img, int $src_x, int $src_y, int $src_w, int $src_h,
	int $dst_x, int $dst_y, int $dst_w, int $dst_h, int $dst_canvas_w = 0, int $dst_canvas_h = 0,
	int $bg_color_red = 0, int $bg_color_green = 0, int $bg_color_blue = 0)
	{
		// Dimensions de l'image finale.
		$dst_canvas_w = ($dst_canvas_w == 0) ? $dst_w : $dst_canvas_w;
		$dst_canvas_h = ($dst_canvas_h == 0) ? $dst_h : $dst_canvas_h;
		$dst_img = imagecreatetruecolor((int) round($dst_canvas_w), (int) round($dst_canvas_h));

		// Conservation de la transparence.
		self::transparency($dst_img);

		// Couleur de fond.
		$bg_color = imagecolorallocate($dst_img,
			$bg_color_red, $bg_color_green, $bg_color_blue);
		imagefill($dst_img, 0, 0, $bg_color);

		// Redimensionnement.
		imagecopyresampled($dst_img, $src_img, $dst_x, $dst_y, $src_x, $src_y,
			(int) round($dst_w), (int) round($dst_h), (int) round($src_w), (int) round($src_h));

		imagedestroy($src_img);

		return $dst_img;
	}

	/**
	 * Permet de conserver la transparence lors de transformations sur une image.
	 *
	 * @param resource $image
	 *   Ressource de l'image.
	 *
	 * @return void
	 */
	public static function transparency(&$image): void
	{
		if (CONF_GD_TRANSPARENCY
		&& function_exists('imagealphablending')
		&& function_exists('imagesavealpha')
		&& is_resource($image))
		{
			imagealphablending($image, FALSE);
			imagesavealpha($image, TRUE);
		}
	}
}
?>