<?php
declare(strict_types = 1);

/**
 * Gestion des catégories parentes.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class Parents
{
	/**
	 * Récupère les informations utiles des
	 * catégories $categories_id.
	 *
	 * @param array $categories_id
	 *
	 * @return array
	 */
	public static function getInfos(array $categories_id): array
	{
		static $cache = [];

		$data = [];
		foreach ($categories_id as &$cat_id)
		{
			if (array_key_exists($cat_id, $cache))
			{
				$data[$cat_id] = $cache[$cat_id];
			}
			else
			{
				$sql = "SELECT cat_id,
							   cat_name,
							   cat_url,
							   cat_votable,
							   cat_commentable,
							   cat_downloadable,
							   cat_uploadable,
							   cat_creatable,
							   cat_a_subalbs + cat_a_subcats +
							   cat_d_subalbs + cat_d_subcats AS cat_subs,
							   cat_a_subalbs + cat_a_subcats AS cat_a_subs,
							   cat_orderby,
							   cat_filemtime,
							   CASE WHEN cat_filemtime IS NULL
									THEN 'category' ELSE 'album'
									 END AS cat_type
						  FROM {categories}
						 WHERE cat_id IN (" . DB::inInt($categories_id) . ")
					  ORDER BY LENGTH(cat_path) ASC";
				if (!DB::execute($sql))
				{
					return [];
				}
				$data = DB::fetchAll('cat_id');

				foreach ($data as $cat_id => &$i)
				{
					if (!array_key_exists($cat_id, $cache))
					{
						$cache[$cat_id] = $i;
					}
				}

				return $data;
			}
		}

		return $data;
	}

	/**
	 * Définit les réglages des objets $objects_infos
	 * en fonction de leurs parents.
	 *
	 * @param array $objects_infos
	 *
	 * @return void
	 */
	public static function settings(array &$objects_infos): void
	{
		foreach ($objects_infos as &$i)
		{
			$parents = Category::getParentsIdArray($i['cat_parents']);
			array_shift($parents);
			$i['parents_id'] = $parents;
			foreach ($parents as &$id)
			{
				$parents_id[$id] = 1;
			}
		}
		if (!empty($parents_id))
		{
			$parents_infos = self::getInfos(array_keys($parents_id));
		}
		$perms = ['commentable', 'creatable', 'downloadable', 'uploadable', 'votable'];
		foreach ($objects_infos as &$i)
		{
			foreach ($perms as &$p)
			{
				$i['parent_' . $p] = '1';
			}

			foreach ($i['parents_id'] as &$id)
			{
				// Permissions.
				if ($id)
				{
					foreach ($perms as &$p)
					{
						if ($parents_infos[$id]['cat_' . $p] != '1')
						{
							$i['parent_' . $p] = '0';
						}
					}
				}
			}

			unset($i['parents_id']);
		}
	}

	/**
	 * Met à jour les informations suivantes des catégories $parents_id :
	 *
	 * - Choisi une nouvelle vignette lorsque le fichier correspondant
	 *   à la vignette de la catégorie a été supprimé ou désactivé.
	 *
	 * - Indique une catégorie comme vide lorsqu'aucun fichier de
	 *   cette catégorie n'est activé.
	 *
	 * - Active une catégorie lorsqu'au moins un fichier activé se
	 *   trouve dans cette catégorie.
	 *
	 * - Met à jour la date d'ajout du dernier fichier de la catégorie.
	 *
	 * - Incrémente/décrémente le nombre d'albums activés/désactivés pour
	 *   les catégories parentes si un album est désactivé mais qu'il
	 *   contient des fichiers activés.
	 *
	 * @param array $parents_id
	 *
	 * @return bool
	 *   TRUE si succès, FALSE si échec.
	 */
	public static function updateInfos(array $parents_id): bool
	{
		// On récupère les informations utiles des catégories.
		$sql = "SELECT cat_id,
					   cat_path,
					   thumb_id,
					   cat_tb_params,
					   cat_parents,
					   cat_a_albums,
					   cat_d_albums,
					   cat_a_images + cat_a_videos AS cat_a_items,
					   cat_d_images + cat_d_videos AS cat_d_items,
					   cat_crtdt,
					   cat_status,
					   cat_filemtime,
					   (SELECT item_status
					      FROM {items}
						 WHERE thumb_id = item_id
						   AND item_path LIKE cat_path || '/%') AS item_status
				  FROM {categories}
				 WHERE cat_id IN (" . DB::inInt($parents_id) . ")
			  ORDER BY LENGTH(cat_parents) DESC";
		if (!DB::execute($sql))
		{
			return FALSE;
		}
		$categories = DB::fetchAll('cat_id');

		// La catégorie 1 doit toujours être en dernier.
		$cat_one = $categories[1];
		unset($categories[1]);
		$categories[1] = $cat_one;

		// On traite chaque catégorie.
		$albums_add = 0;
		foreach ($categories as &$i)
		{
			$cat_path = ($i['cat_id'] > 1) ? DB::likeEscape($i['cat_path']) . '/%' : '%';
			$cols =
			[
				'thumb_id' => $i['thumb_id'],
				'cat_tb_params' => $i['cat_tb_params'],
				'cat_a_albums' => $i['cat_a_albums'] + $albums_add,
				'cat_d_albums' => $i['cat_d_albums'] - $albums_add,
				'cat_status' => ($i['cat_a_items'] > 0) ? '1' : $i['cat_status'],
				'cat_id' => $i['cat_id']
			];

			if ($i['cat_id'] == 1)
			{
				goto lastadddt;
			}

			// Si la catégorie ne contient aucun fichier,
			// on supprime la référence à une vignette.
			if (($i['cat_a_items'] + $i['cat_d_items']) == 0)
			{
				$cols['thumb_id'] = -1;
				$cols['cat_tb_params'] = NULL;
				goto lastadddt;
			}

			// S'il s'agit d'une vignette d'une image externe,
			// alors on laisse cette vignette.
			if ($i['thumb_id'] == 0)
			{
				goto lastadddt;
			}

			// On choisi une nouvelle vignette pour la catégorie si :
			// - Le fichier de la vignette de la catégorie n'existe pas dans
			//   cette catégorie.
			// - Ou le fichier correspondant à cette vignette est désactivé
			//   alors que la catégorie contient des fichiers activés.
			if (!in_array($i['item_status'], ['0', '1'])
			|| ($i['cat_a_items'] > 0 && $i['item_status'] === '0'))
			{
				$sql = "SELECT item_id
						  FROM {items}
						 WHERE item_path LIKE ?
					  ORDER BY item_status DESC,
							   item_id DESC
						 LIMIT 1";
				if (!DB::execute($sql, $cat_path))
				{
					return FALSE;
				}
				$cols['thumb_id'] = DB::fetchVal();

				// On supprime les coordonnées de rognage
				// de la vignette précédente.
				$cols['cat_tb_params'] = NULL;
			}

			// Date d'ajout du dernier fichier.
			lastadddt:
			$sql = "SELECT item_adddt
					  FROM {items}
					 WHERE item_path LIKE ?
				  ORDER BY item_status DESC,
						   item_adddt DESC
					 LIMIT 1";
			if (!DB::execute($sql, $cat_path))
			{
				return FALSE;
			}
			$cols['cat_lastadddt'] = DB::fetchVal();

			$params[] = $cols;

			// Si c'est un album, qu'il est désactivé et qu'il contient des
			// fichiers activés, alors on indique d'incrémenter le nombre
			// d'albums activés et de décrémenter le nombre d'albums désactivés
			// pour toutes les catégories parentes.
			if ($i['cat_filemtime'] && $i['cat_status'] == 0 && $i['cat_a_items'] > 0)
			{
				$albums_add = 1;
			}
		}

		if (empty($params))
		{
			return TRUE;
		}

		$sql = "UPDATE {categories}
				   SET thumb_id = :thumb_id,
				       cat_tb_params = :cat_tb_params,
					   cat_a_albums = :cat_a_albums,
					   cat_d_albums = :cat_d_albums,
					   cat_lastadddt = :cat_lastadddt,
				       cat_status = :cat_status
				 WHERE cat_id = :cat_id";
		return DB::execute($sql, $params);
	}

	/**
	 * Mise à jour des statistiques des catégories parentes.
	 *
	 * @param array $update
	 *   Statistiques à additionner ou soustraire aux catégories.
	 * @param string $a_sign
	 *   Additionne ('+') ou soustrait ('-') les informations des
	 *   fichiers activés. Chaîne vide si aucune modification souhaitée.
	 * @param string $d_sign
	 *   Additionne ('+') ou soustrait ('-') les informations des
	 *   fichiers désactivés. Chaîne vide si aucune modification souhaitée.
	 * @param array $parents_id
	 *   Identifiants des catégories parentes à mettre à jour.
	 *
	 * @return bool
	 *   TRUE si succès, FALSE si échec.
	 */
	public static function updateStats(array $update,
	string $a_sign, string $d_sign, array $parents_id): bool
	{
		foreach(['a' => $a_sign, 'd' => $d_sign] as $l => $s)
		{
			if (!in_array($s, ['+', '-']))
			{
				continue;
			}

			$albums = (int) $update[$l]['albums'];
			$comments = (int) $update[$l]['comments'];
			$favorites = (int) $update[$l]['favorites'];
			$hits = (int) $update[$l]['hits'];
			$images = (int) $update[$l]['images'];
			$rating = (float) $update[$l]['rating'];
			$size = (int) $update[$l]['size'];
			$videos = (int) $update[$l]['videos'];
			$votes = (int) $update[$l]['votes'];

			$sql_set[] = "cat_{$l}_albums = CASE
				WHEN cat_filemtime IS NULL
				THEN cat_{$l}_albums $s $albums
				ELSE 0
				 END";
			$sql_set[] = "cat_{$l}_comments = cat_{$l}_comments $s $comments";
			$sql_set[] = "cat_{$l}_favorites = cat_{$l}_favorites $s $favorites";
			$sql_set[] = "cat_{$l}_hits = cat_{$l}_hits $s $hits";
			$sql_set[] = "cat_{$l}_images = cat_{$l}_images $s $images";
			if ($rating)
			{
				$sql_set[] = "cat_{$l}_rating = CASE
					WHEN cat_{$l}_votes $s $votes > 0
					THEN ((cat_{$l}_rating * cat_{$l}_votes) $s ($rating * $votes))
					   / (cat_{$l}_votes $s $votes)
					ELSE 0
					 END";
			}
			$sql_set[] = "cat_{$l}_size = cat_{$l}_size $s $size";
			$sql_set[] = "cat_{$l}_videos = cat_{$l}_videos $s $videos";
			$sql_set[] = "cat_{$l}_votes = cat_{$l}_votes $s $votes";
		}

		$sql = "UPDATE {categories}
				   SET " . implode(', ', $sql_set) . "
		         WHERE cat_id IN (" . DB::inInt($parents_id) . ")";
		return DB::execute($sql);
	}

	/**
	 * Met à jour le nombre de sous-albums et de sous-catégories
	 * pour les catégories $categories_id.
	 *
	 * @param array $categories_id
	 *
	 * @return bool
	 *   TRUE si succès, FALSE si échec.
	 */
	public static function updateSubCats(array $categories_id): bool
	{
		$sql = "SELECT parent_id, cat_status, cat_filemtime
				  FROM {categories}
				 WHERE parent_id IN (" . DB::inInt($categories_id) . ")
				   AND cat_id > 1";
		if (!DB::execute($sql))
		{
			return FALSE;
		}
		$cat_infos = DB::fetchAll();

		foreach ($categories_id as &$id)
		{
			$params[$id] = [0, 0, 0, 0, $id];
		}
		foreach ($cat_infos as &$d)
		{
			$i = 0;
			foreach ([1, 0] as $s)
			{
				if ($d['cat_status'] == $s)
				{
					$params[$d['parent_id']][$i + (int) (!$d['cat_filemtime'])]++;
				}
				$i = 2;
			}
		}

		$sql = "UPDATE {categories}
				   SET cat_a_subalbs = ?, cat_a_subcats = ?,
					   cat_d_subalbs = ?, cat_d_subcats = ?
				 WHERE cat_id = ?";
		return DB::execute($sql, array_values($params));
	}
}
?>