<?php
declare(strict_types = 1);

/**
 * Code SQL commun.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class SQL
{
	/**
	 * Retourne la clause ORDER BY d'une requête SQL sur la table {categories}.
	 *
	 * @param string $str
	 *   Contenu de la clause ORDER BY.
	 * @param array $i
	 *   Informations de la catégorie.
	 *
	 * @return string
	 */
	public static function catOrderBy(string $str, array &$i = []): string
	{
		if (!self::_checkOrderBy($str, 'category'))
		{
			$str = 'cat_name ASC';
		}

		$cat_path = '';
		if (strpos($str, 'cat_path_num') !== FALSE)
		{
			if (isset($i['cat_path'])
			&& preg_match('`^[-_a-z0-9]+$`i', $i['cat_path']))
			{
				$cat_path = $i['cat_path'];
			}
		}

		$items_replace =
		[
			'cat_a_items' => '(cat_a_images + cat_a_videos)',
			'cat_name_num' => 'LOWER(cat_name)+0',
			'cat_name' => 'LOWER(cat_name)',
			'cat_path_num' => $cat_path
				? "REPLACE(cat_path, '$cat_path/', '')+0"
				: "LOWER(cat_path)",
			'cat_path' => 'LOWER(cat_path)'
		];

		return str_replace(array_keys($items_replace), array_values($items_replace), $str);
	}

	/**
	 * Retourne la partie de la clause WHERE ou SELECT d'une requête
	 * SQL permettant de déterminer si l'accès à une catégorie
	 * protégée par mot de passe est autorisée avec le jeton de
	 * session que possède l'utilisateur.
	 *
	 * Valeurs possibles pour password_auth :
	 *	   1 : Aucun mot de passe sur la catégorie.
	 *     2 : Mot de passe présent sur la catégorie et utilisateur
	 *         possédant un identifiant de session valide pour cette
	 *         catégorie.
	 *  NULL : Mot de passe présent sur la catégorie et utilisateur
	 *         ne possédant pas d'identifiant de session valide pour
	 *         cette catégorie.
	 *
	 * @param string $clause
	 *   'where' ou 'select'.
	 *
	 * @return string
	 */
	public static function catPassword(string $clause = 'where'): string
	{
		if ($session_token = Auth::getCookieSessionToken())
		{
			$sql = "SELECT 2
					  FROM {sessions} AS s
				 LEFT JOIN {sessions_categories} AS sc USING (session_id)
				 LEFT JOIN {passwords} AS p USING (cat_id)
					 WHERE cat.password_id = p.password_id
					   AND session_token = '$session_token'
					   AND session_expire > NOW()";
		}
		else
		{
			$sql = "NULL";
		}

		if ($clause == 'select')
		{
			return "CASE WHEN cat.password_id IS NULL THEN 1 ELSE ($sql) END AS password_auth";
		}
		else
		{
			return "(cat.password_id IS NULL OR ($sql) = 2)";
		}
	}

	/**
	 * Retourne la partie de la clause WHERE d'une requête SQL qui gère
	 * l'accès aux catégories en fonction des permissions d'accès aux
	 * catégories de l'utilisateur connecté ou faisant partie du groupe
	 * $id (si spécifié) avec liste de blocage $list (si spécifiée).
	 *
	 * @param int $id
	 *   Identifiant du groupe.
	 *   Si non fourni, utilise l'identifiant du groupe de
	 *   l'utilisateur connecté.
	 * @param string $list
	 *   Liste de blocage : 'black' ou 'white'.
	 *   Si non fournie, utilise soit la liste de blocage définie
	 *   dans le groupe $id, soit celle de l'utilisateur connecté.
	 *
	 * @return string
	 */
	public static function catPerms(int $id = 0, string $list = ''): string
	{
		static $sql_perms = [];

		// Identifiant de groupe de l'utilisateur connecté si non fourni.
		if ($id)
		{
			$group_id = $id;
		}
		else
		{
			$group_id = (int) (Auth::$infos['group_id'] ?? 2);
		}

		// Si les permissions ont déjà été récupérée,
		// on retourne le code SQL déjà mémorisé.
		if (isset($sql_perms[$group_id]))
		{
			return $sql_perms[$group_id];
		}
		$sql_perms[$group_id] = '1=1';

		$fail = function()
		{
			trigger_error('Missing permissions.', E_USER_ERROR);
			die('Error: Missing permissions.');
		};

		// On n'ajoute pas les permissions pour la galerie
		// si la gestion de membres n'est pas activée.
		if (!Config::$params['users'])
		{
			return $sql_perms[$group_id];
		}

		// Si l'utilisateur est un administrateur,
		// inutile d'aller plus loin.
		if (Auth::$isAdmin)
		{
			return $sql_perms[$group_id];
		}

		// Récupération des permissions d'accès aux catégories.
		$sql = "SELECT ga.*,
					   cat.cat_parents
				  FROM {groups_permissions} AS ga
			 LEFT JOIN {categories} AS cat
				    ON ga.cat_id = cat.cat_id
				 WHERE group_id = ?";
		if (!DB::execute($sql, $group_id))
		{
			$fail();
		}
		$perms = DB::fetchAll();

		// Classement des catégories par type de permission.
		$perms_list = ['black' => [], 'white' => []];
		$cat_parents = [];
		foreach ($perms as &$i)
		{
			$perms_list[$i['perm_list']][] = $i['cat_id'];
			$cat_parents[$i['cat_id']] = $i['cat_parents'];
		}

		// Type de liste d'accès aux catégories.
		if (!$list)
		{
			if ($id)
			{
				$sql = "SELECT group_perms FROM {groups} WHERE group_id = ?";
				if (!DB::execute($sql, $group_id) || !$group_perms = DB::fetchVal())
				{
					$fail();
				}
				$group_perms = Utility::jsonDecode($group_perms);
				if (!is_array($group_perms))
				{
					$fail();
				}
				$list = $group_perms['perm_list'];
			}
			else
			{
				$list = Auth::$groupPerms['perm_list'];
			}
		}

		$sep = Category::PARENTS_SEPARATOR;

		// Liste noire.
		if ($list == 'black')
		{
			$list = $perms_list['black'];
			if (!$list)
			{
				return $sql_perms[$group_id];
			}
			$sql_perms[$group_id] = "(cat.cat_id NOT IN (" . DB::inInt($list) . ")";
			foreach ($list as &$cat_id)
			{
				$sql_perms[$group_id]
					.= " AND cat_parents NOT LIKE '%$sep" . (int) $cat_id . "$sep%'";
			}
		}

		// Liste blanche.
		else
		{
			$list = $perms_list['white'];
			if ($list)
			{
				$sql_perms[$group_id] = "(cat.cat_id IN (1, " . DB::inInt($list) . ")";
				$parents_id = [];
				foreach ($list as &$cat_id)
				{
					$sql_perms[$group_id]
						.= " OR cat_parents LIKE '%$sep" . (int) $cat_id . "$sep%'";

					// Catégories parentes qui doivent être accessibles même si elles
					// ne sont pas en liste blanche.
					if ($p = substr($cat_parents[$cat_id], 2, -1))
					{
						$parents_id = array_merge($parents_id, explode($sep, $p));
					}
				}
				if ($parents_id)
				{
					$sql_perms[$group_id] = str_replace(
						"(cat.cat_id IN (",
						"(cat.cat_id IN (" . DB::inInt(array_unique($parents_id)) . ", ",
						$sql_perms[$group_id]
					);
				}
			}
			else
			{
				$sql_perms[$group_id] = "(cat.cat_id IN (1)";
			}
		}

		$sql_perms[$group_id] .= ')';

		return $sql_perms[$group_id];
	}

	/**
	 * Retourne un intervalle de date pour les filtres par date.
	 *
	 * @param string $column
	 *   Nom de la colonne.
	 * @param string $date
	 *   Date au format YYYY-MM-DD ou YYYY-MM ou YYYY.
	 *
	 * @return array
	 */
	public static function dateInterval(string $column, string $date): array
	{
		switch (strlen($date))
		{
			// Jour.
			case 10 :
				$date_start = $date;
				$date_end = $date;
				break;

			// Mois.
			case 7 :
				$date_start = $date . '-01';
				$date_end = $date . '-31';
				break;

			// Année.
			case 4 :
				$date_start = $date . '-01-01';
				$date_end = $date . '-12-31';
				break;
		}

		return
		[
			'sql' => "`$column` >= :filter_1 AND `$column` <= :filter_2",
			'params' =>
			[
				'filter_1' => $date_start . ' 00:00:00',
				'filter_2' => $date_end . ' 23:59:59'
			]
		];
	}

	/**
	 * Retourne la clause FROM correspondant
	 * aux critères de recherche des fichiers à récupérer.
	 *
	 * @param string $filter
	 *
	 * @return string
	 */
	public static function itemsFrom(string $filter): string
	{
		switch ($filter)
		{
			case 'camera-brand' :
			case 'camera-model' :
				return " LEFT JOIN {cameras_items} AS cam_i
								ON i.item_id = cam_i.item_id
						 LEFT JOIN {cameras_models} AS cam_m
								ON cam_i.camera_model_id = cam_m.camera_model_id
						 LEFT JOIN {cameras_brands} AS cam_b
								ON cam_m.camera_brand_id = cam_b.camera_brand_id";

			case 'tag' :
				return " LEFT JOIN {tags_items} AS ti
								ON i.item_id = ti.item_id";

			case 'user-favorites' :
				return " LEFT JOIN {favorites} AS fav
								ON i.item_id = fav.item_id";

			default :
				return "";
		}
	}

	/**
	 * Retourne la clause ORDER BY d'une requête SQL sur la table {items}.
	 *
	 * @param string $filter
	 *   Nom du filtre.
	 * @param string $str
	 *   Contenu de la clause ORDER BY.
	 * @param array $i
	 *   Informations de la catégorie.
	 *
	 * @return string
	 */
	public static function itemsOrderBy(string $filter,
	string $str, array &$i = []): string
	{
		if (!self::_checkOrderBy($str, 'album'))
		{
			$str = 'item_adddt DESC';
		}

		$cat_path = '';
		if (strpos($str, 'item_path_num') !== FALSE)
		{
			if (isset($i['cat_path'])
			&& preg_match('`^[-_a-z0-9]+$`i', $i['cat_path']))
			{
				$cat_path = $i['cat_path'];
			}
		}

		switch ($filter)
		{
			// Sans filtre.
			case '' :
				$items_replace =
				[
					'item_name_num' => "LOWER(item_name)+0",
					'item_name' => "LOWER(item_name)",
					'item_path_num' => $cat_path
						? "REPLACE(item_path, '$cat_path/', '')+0"
						: "LOWER(item_path)",
					'item_path' => "LOWER(item_path)",
					'item_size' => "(item_width*item_height)"
				];
				$sql = str_replace(
					array_keys($items_replace),
					array_values($items_replace),
					$str
				);
				break;

			// Nombre de commentaires.
			case 'comments' :
				$sql = "item_comments DESC, i.item_id DESC";
				break;

			// Nombre de favoris.
			case 'favorites' :
				$sql = "item_favorites DESC, i.item_id DESC";
				break;

			// Favoris d'un utilisateur.
			case 'user-favorites' :
				$sql = "fav.fav_date DESC, i.item_id DESC";
				break;

			// Nombre de vues.
			case 'views' :
				$sql = "item_hits DESC, i.item_id DESC";
				break;

			// Note moyenne.
			case 'votes' :
				$sql = "item_rating DESC, item_votes DESC, i.item_id DESC";
				break;

			default :
				$sql = "i.item_id DESC";
				break;
		}

		return $sql;
	}

	/**
	 * Retourne la clause WHERE correspondant
	 * aux critères de recherche des fichiers à récupérer.
	 *
	 * @param string $filter
	 * @param string $value
	 *
	 * @return mixed
	 */
	public static function itemsWhere(string $filter, string $value)
	{
		$sql = "";
		$params = ['filter' => (int) $value];

		switch ($filter)
		{
			case 'camera-brand' :
				$sql = " cam_b.camera_brand_id = :filter";
				break;

			case 'camera-model' :
				$sql = " cam_i.camera_model_id = :filter";
				break;

			case 'comments' :
				$sql = " item_comments > 0";
				$params = [];
				break;

			case 'date-added' :
				return self::dateInterval('item_adddt', $value);

			case 'date-created' :
				return self::dateInterval('item_crtdt', $value);

			case 'favorites' :
				$sql = " item_favorites > 0";
				$params = [];
				break;

			case 'images' :
				$types = DB::inInt(Item::IMAGE_TYPES);
				$sql = " i.item_type IN ($types)";
				$params = [];
				break;

			case 'items' :
				$sql = " 1";
				$params = [];
				break;

			case 'recent-images' :
			case 'recent-items' :
			case 'recent-videos' :
				$params = ['filter' => date('Y-m-d H:i:s', Item::getRecentTimestamp())];
				$types = "";
				switch ($filter)
				{
					case 'recent-images' :
						$types = DB::inInt(Item::IMAGE_TYPES);
						$types = " AND i.item_type IN ($types)";
						break;

					case 'recent-videos' :
						$types = DB::inInt(Item::VIDEO_TYPES);
						$types = " AND i.item_type IN ($types)";
						break;
				}
				$sql = " item_adddt > :filter$types";
				break;

			case 'tag' :
				$sql = " ti.tag_id = :filter";
				break;

			case 'user-favorites' :
				$sql = " fav.user_id != 2 AND fav.user_id = :filter";
				break;

			case 'user-images' :
				$types = DB::inInt(Item::IMAGE_TYPES);
				$sql = " i.user_id != 2 AND i.user_id = :filter AND i.item_type IN ($types)";
				break;

			case 'user-items' :
				$sql = " i.user_id != 2 AND i.user_id = :filter";
				break;

			case 'user-videos' :
				$types = DB::inInt(Item::VIDEO_TYPES);
				$sql = " i.user_id != 2 AND i.user_id = :filter AND i.item_type IN ($types)";
				break;

			case 'videos' :
				$types = DB::inInt(Item::VIDEO_TYPES);
				$sql = " i.item_type IN ($types)";
				$params = [];
				break;

			case 'views' :
				$sql = " item_hits > 0";
				$params = [];
				break;

			case 'votes' :
				$sql = " item_votes > 0";
				$params = [];
				break;
		}

		if (!$sql)
		{
			return FALSE;
		}

		return ['sql' => $sql, 'params' => $params];
	}



	/**
	 * Vérifie la validité d'une clause ORDER BY appliquée à une catégorie.
	 *
	 * @param string $orderby
	 * @param string $type
	 *   Type de catégorie : 'album' ou 'category'.
	 *
	 * @return bool
	 */
	private static function _checkOrderBy(string $orderby, string $type): bool
	{
		if (!preg_match('`^(?:[\sa-z_]{3,30}(?:,\s)?){1,3}$`i', $orderby))
		{
			return FALSE;
		}

		$orderby = explode(',', $orderby);
		$params = Category::getOrderByParams()[$type];
		foreach ($orderby as &$e)
		{
			$e = explode(' ', trim($e));
			if (!isset($params[$e[0]]) || !in_array($e[1], ['ASC', 'DESC']))
			{
				return FALSE;
			}
		}

		return TRUE;
	}
}
?>