<?php
declare(strict_types = 1);

/**
 * Classe mère pour toute la galerie.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class Gallery
{
	/**
	 * Liste des catégories.
	 *
	 * @var array
	 */
	public static $categories = [];

	/**
	 * Paramètres de recherche.
	 *
	 * @var object
	 */
	public static $search;



	/**
	 * Modifie les statistiques de catégories en fonction
	 * des permissions d'accès de l'utilisateur.
	 *
	 * @param array $cat_infos
	 *
	 * @return bool
	 *   Retourne FALSE en cas d'erreur.
	 */
	public static function changeCatStats(array &$cat_infos): bool
	{
		static $reduce_cats;
		if ($reduce_cats === [])
		{
			return TRUE;
		}

		// Catégories parentes.
		$sep = Category::PARENTS_SEPARATOR;
		if ($multiple = !isset($cat_infos['cat_id']))
		{
			$key = key($cat_infos);
			$cat_id = $cat_infos[$key]['cat_id'];
			$cat_parents = $cat_infos[$key]['cat_parents'];
			$categories = &$cat_infos;
		}
		else
		{
			$cat_id = $cat_infos['cat_id'];
			$cat_parents = $cat_infos['cat_parents'];
			$categories = [$cat_id => &$cat_infos];
		}
		$cat_parents = $cat_parents . ($cat_id > 1 ? $cat_id . $sep : '');

		if (!$reduce_cats)
		{
			$cat_perms = SQL::catPerms();

			// On "inverse" les permissions d'accès aux catégories.
			if (strlen($cat_perms) < 9)
			{
				$cat_perms = '1=2';
			}
			elseif (Auth::$groupPerms['perm_list'] == 'black')
			{
				$cat_perms = preg_replace('` (AND|OR) cat_parents.*$`', ')', $cat_perms);
				$cat_perms = str_replace('NOT ', '', $cat_perms);
			}
			else
			{
				$cat_perms = str_replace(
					['IN', 'LIKE', 'OR'], ['NOT IN', 'NOT LIKE', 'AND'], $cat_perms
				);
			}
			$cat_password = preg_replace(
				'`\(cat\.password_id IS NULL OR ([^$]+) = \d\)`',
				'(cat.password_id IS NOT NULL AND $1 IS NULL)',
				SQL::catPassword()
			);

			// On récupère les infos utiles de toutes les catégories protégées
			// non déverrouillées par les permissions de l'utilisateur.
			$sql = "SELECT cat_id,
						   parent_id,
						   cat_parents,
						   cat_status,
						   cat_filemtime,
						   cat_a_albums,
						   cat_a_comments,
						   cat_a_favorites,
						   cat_a_hits,
						   cat_a_images,
						   cat_a_rating,
						   cat_a_size,
						   cat_a_subalbs,
						   cat_a_subcats,
						   cat_a_videos,
						   cat_a_votes
					  FROM {categories} AS cat
					 WHERE cat_parents LIKE ?
					   AND cat.cat_id > 1
					   AND cat_status = '1'
					   AND thumb_id > 0
					   AND (%s OR %s)
				  ORDER BY LENGTH(cat_parents) ASC";
			$sql = sprintf($sql, $cat_perms, $cat_password);
			if (!DB::execute($sql, [$cat_parents . '%']))
			{
				return FALSE;
			}
			$reduce_cats = DB::fetchAll('cat_id');
		}
		if (!$reduce_cats)
		{
			return TRUE;
		}

		// Réduction des stats.
		$temp_parents = [];
		foreach ($categories as $id => &$i)
		{
			foreach ($reduce_cats as $cat_id => &$s)
			{
				if ($multiple && strpos($s['cat_parents'] . $cat_id,
				$i['cat_parents'] . $id . $sep, 0) === FALSE)
				{
					continue;
				}

				// Il ne faut pas retrancher les stats pour une
				// sous-catégorie d'une catégorie qui l'a déjà été !
				foreach ($temp_parents as &$p)
				{
					if (strpos($s['cat_parents'] . $s['cat_id'], $p . $sep, 0) !== FALSE)
					{
						continue 2;
					}
				}
				$temp_parents[] = $s['cat_parents'] . $s['cat_id'];

				// Correction pour le nombre d'albums.
				$s['cat_a_albums'] = ($s['cat_filemtime'] === NULL) ? $s['cat_a_albums'] : 1;

				// On réduit le nombre d'albums.
				$i['cat_a_albums'] -= ($s['cat_filemtime'] === NULL) ? $s['cat_a_albums'] : 1;

				// On recalcule la note moyenne.
				if ($i['cat_a_votes'] - $s['cat_a_votes'] > 0)
				{
					$i['cat_a_rating'] =
						(($i['cat_a_rating'] * $i['cat_a_votes'])
						- ($s['cat_a_rating'] * $s['cat_a_votes']))
						/ ($i['cat_a_votes'] - $s['cat_a_votes']);
				}
				else
				{
					$i['cat_a_rating'] = 0;
				}

				// Réduction des autres stats.
				$i['cat_a_comments'] -= $s['cat_a_comments'];
				$i['cat_a_favorites'] -= $s['cat_a_favorites'];
				$i['cat_a_hits'] -= $s['cat_a_hits'];
				$i['cat_a_images'] -= $s['cat_a_images'];
				$i['cat_a_size'] -= $s['cat_a_size'];
				$i['cat_a_videos'] -= $s['cat_a_videos'];
				$i['cat_a_votes'] -= $s['cat_a_votes'];
			}
		}

		return TRUE;
	}

	/**
	 * Détermine si l'utilisateur possède un jeton de session lui
	 * permettant d'accèder à une catégorie. Si ce n'est pas le
	 * cas, on redirige vers la page de demande de mot de passe.
	 *
	 * @params array $i
	 *   Informations de la catégorie.
	 *
	 * @return bool
	 */
	public static function checkCookiePassword(array &$i): bool
	{
		// La catégorie ne possède aucun mot de passe.
		if ($i['password_auth'] == 1)
		{
			return TRUE;
		}

		// La catégorie est protégée par un mot de passe :
		// 1. L'utilisateur a entré le bon mot de passe.
		if ($i['password_auth'] == 2)
		{
			// Mise à jour de la date d'expiration de la session,
			// mais seulement si l'utilisateur n'est pas authentifié.
			if (!Auth::$connected)
			{
				$session_expire = (int) CONF_SESSION_EXPIRE;
				$sql = "UPDATE {sessions}
						   SET session_expire = DATE_ADD(NOW(), INTERVAL $session_expire SECOND)
						 WHERE session_token = ?";
				DB::execute($sql, Auth::getCookieSessionToken());
			}
			return TRUE;
		}

		// 2. Mauvais mot de passe : on redirige.
		if ($_GET['section'] == 'item')
		{
			App::redirect('password/item/' . $i['item_id']);
		}
		else
		{
			App::redirect('password/' . $i['cat_type'] . '/' . $i['cat_id']);
		}
		return FALSE;
	}

	/**
	 * Gestion des filtres.
	 *
	 * @return void
	 */
	public static function filters(): void
	{
		// Filtres.
		if (isset($_GET['filter']))
		{
			// Template : paramètres de chaque section.
			switch ($_GET['section'])
			{
				case 'album' :
				case 'category' :
				case 'item' :
					if (!Template::filter((int) $_GET['filter_value'], $_GET['q_filterless']))
					{
						self::notFound();
					}
					break;
			}
		}
	}

	/**
	 * Initialisation.
	 *
	 * @return void
	 */
	public static function start(): void
	{
		// On redirige vers le bon URL si l'URL demandé
		// ne correspond pas à l'option sur l'URL rewriting choisie.
		if (isset($_GET['q']) && isset($_SERVER['REQUEST_URI']))
		{
			$gallery_base_url = (CONF_URL_REWRITE)
				? App::getURL('')
				: substr(App::getURL('1'), 0, -1);
			if (preg_match('`/\?q=`', $_SERVER['REQUEST_URI'])
			 != preg_match('`/\?q=`', $gallery_base_url))
			{
				App::redirect($_GET['q'], 301);
			}
		}

		// Changement de page.
		if (isset($_POST['page'])
		&& preg_match('`^\d{1,12}$`', $_POST['page']) && $_POST['page'] > 0)
		{
			$anchor = '';
			if (isset($_POST['anchor']) && preg_match('`^[a-z0-9_-]{1,30}$`i', $_POST['anchor']))
			{
				$anchor = '#' . $_POST['anchor'];
			}
			App::redirect($_GET['q'] . '/page/' . $_POST['page'] . $anchor);
		}

		// Connexion à la base de données.
		if (!DB::connect())
		{
			die('Impossible de se connecter à la base de données.'
				. '<br/>Unable to connect to the database.');
		}

		// Récupération de la configuration.
		Config::getDBParams();

		// Gestion des paramètres GET.
		self::_request();

		// Initialisation du template.
		Template::init();

		// Gestion de membres.
		Template::set('users', (int) Config::$params['users']);

		// La galerie est-elle fermée ?
		if (Config::$params['gallery_closed'])
		{
			Template::set('lang_selected', Auth::$lang);
			Template::set('page_title', function(){ return __('Galerie fermée'); });
			return;
		}

		// Authentification utilisateur par cookie.
		if (Auth::cookie() && (Config::$params['users'] || Auth::$isAdmin))
		{
			Template::set('user',
			[
				'admin' => Auth::$isAdmin,
				'auth' => TRUE,
				'avatar_source' => Avatar::getURL(Auth::$id, (bool) Auth::$infos['user_avatar']),
				'id' => Auth::$id,
				'lang' => &Auth::$lang,
				'nickname' => Auth::$nickname,
				'perms' => Auth::$groupPerms,
				'tz' => &Auth::$tz
			]);
		}

		// Invité.
		else
		{
			// Authentification par formulaire.
			if (Config::$params['users'] && isset($_POST['connection'])
			&& isset($_POST['login']) && isset($_POST['password'])
			&& Auth::form($_POST['login'], $_POST['password'], isset($_POST['remember'])))
			{
				App::redirect();
			}

			// Paramètres par défaut.
			Template::set('user',
			[
				'admin' => Auth::$isAdmin,
				'auth' => FALSE,
				'avatar_source' => Avatar::getURL(2, FALSE),
				'id' => 2,
				'login' => NULL,
				'perms' => Auth::$groupPerms
			]);

			// Galerie autorisée seulement pour les utilisateurs enregistrés ?
			if (Config::$params['users_only_members']
			&& !in_array($_GET['section'], ['forgot-password', 'login',
			'new-password', 'register', 'validation']))
			{
				App::redirect('login');
				die;
			}
		}

		// Localisation.
		$lang_switch = !Auth::$connected && Config::$params['lang_switch'];
		Template::set('lang_default', Config::$params['lang_default']);
		Template::set('lang_switch', []);
		if ($lang_switch)
		{
			$params = Config::$params['lang_params']['langs'];
			$langs = [];
			foreach ($params as $code => &$name)
			{
				$langs[$code] = ['code' => $code, 'name' => $name];
			}
			Template::set('lang_switch', $langs);
		}
		if (isset($_POST['lang']) && ($_GET['section'] == 'user-options' || $lang_switch))
		{
			$lang = $_POST['lang'];
		}
		elseif ($lang_switch)
		{
			$lang = Auth::$prefs->read('lang');
		}
		if (isset($_POST['tz']) && $_GET['section'] == 'user-options')
		{
			$tz = $_POST['tz'];
		}
		L10N::locale($lang ?? '', $tz ?? '');
		Template::set('lang_selected', Auth::$lang);

		// Titre de page.
		Template::set('page_title', function()
		{
			$page = '';
			$separator = ' - ';
			$section_title = '';
			switch ($_GET['section'])
			{
				case '404' :
					$section_title = __('Page non trouvée');
					break;

				case 'album' :
				case 'category' :
					if (Template::$data['category']['id'] > 1)
					{
						$sql_limit = $_GET['section'] == 'album'
							? Config::$params['thumbs_item_nb_per_page']
							: Config::$params['thumbs_cat_nb_per_page'];
						if (Template::$data['objects_count'] > $sql_limit)
						{
							$page = $separator . sprintf(__('Page %s'), $_GET['page']);
						}
						$section_title = Template::$data['category']['title'] . $page;
					}
					break;

				case 'cameras' :
					$section_title = __('Appareils photos');
					break;

				case 'comments' :
					$section_title = __('Commentaires');
					break;

				case 'contact' :
					$section_title = __('Contact');
					break;

				case 'error' :
					$section_title = __('Oups !');
					break;

				case 'forgot-password' :
					$section_title = __('Mot de passe oublié');
					break;

				case 'item' :
					$section_title = Template::$data['item']['title'];
					break;

				case 'language' :
					$section_title = __('Choix de la langue');
					break;

				case 'login' :
					$section_title = __('Connexion');
					break;

				case 'logout' :
					$section_title = __('Déconnexion');
					break;

				case 'members' :
					$section_title = __('Liste des membres');
					break;

				case 'new-password' :
					$section_title = __('Nouveau mot de passe');
					break;

				case 'password' :
					$section_title = __('Mot de passe');
					break;

				case 'register' :
					$section_title = __('Créer un compte');
					break;

				case 'search' :
					$section_title = __('Recherche');
					break;

				case 'search-advanced' :
					$section_title = __('Recherche avancée');
					break;

				case 'tags' :
					$section_title = __('Tags');
					break;

				case 'user' :
					$section_title = Template::$data['member']['nickname'];
					break;

				case 'user-avatar' :
					$section_title = __('Changer d\'avatar');
					break;

				case 'user-options' :
					$section_title = __('Options');
					break;

				case 'user-password' :
					$section_title = __('Changer de mot de passe');
					break;

				case 'user-profile' :
					$section_title = __('Modifier vos informations');
					break;

				case 'user-upload' :
					$section_title = __('Ajout de fichiers');
					break;
			}
			$section_title = $section_title ? $section_title . $separator : '';

			return $section_title . Config::$params['gallery_title'];
		});

		// Texte de pied de page.
		$footer_text = Config::$params['gallery_footer_text'];
		Template::set('footer_text', Utility::isEmpty($footer_text) ? NULL : $footer_text);

		// Filtres.
		App::filtersGET();

		// Moteur de recherche.
		if (Config::$params['search'])
		{
			self::_search();
		}

		// Pages.
		$pages_order = Config::$params['pages_order'];
		$pages_params = Config::$params['pages_params'];
		$pages = [];
		foreach ($pages_order as &$name)
		{
			if (($name == 'tags' && !Config::$params['tags'])
			 || ($name == 'comments' && !Config::$params['comments'])
			 || ($name == 'worldmap' && !Config::$params['geolocation'])
			 || ($name == 'members' && !Config::$params['users']))
			{
				continue;
			}
			if ($pages_params[$name]['status'] == 1)
			{
				$id = 0;
				$link = App::getURL($name);
				$title = L10N::getTextPage($name);
				if (substr($name, 0, 7) == 'custom_')
				{
					$id = str_replace('custom_', '', $name);
					$link = App::getURL('page/' . $id . '-' . $pages_params[$name]['url']);
					$title = $pages_params[$name]['title'];
				}

				switch ($name)
				{
					case 'members' :
						$current = $_GET['section'] == $name
							|| substr($_GET['section'], 0, 4) == 'user';
						break;

					default :
						$current = ($_GET['section'] == 'page' && $id)
							? $_GET['page_id'] == $id
							: $_GET['section'] == $name;
						break;
				}

				$pages[] =
				[
					'current' => $current,
					'link' => $link,
					'name' => $name,
					'title' => $title
				];
			}
		}
		Template::set('menu_pages', $pages);

		// Liens.
		if (Config::$params['links'])
		{
			$links = [];
			if (is_array(Config::$params['links_params']))
			{
				foreach (Config::$params['links_params'] as &$i)
				{
					if ($i['activated'])
					{
						$links[] = $i;
					}
				}
			}
			Template::set('links', $links);
		}

		// Géolocalisation.
		Template::set('geolocation', 0);
		if (Config::$params['geolocation'] && in_array($_GET['section'], ['item', 'worldmap']))
		{
			Template::set('geolocation', 1);
		}

		// Option "rester connecté".
		Template::set('remember', !Auth::$session->read('session_token')
			|| Auth::$session->read('session_expire'));

		// Utilisateurs en ligne.
		if (Config::$params['users'] && Config::$params['users_online']
		&& (int) ($s = Config::$params['users_online_duration']) > 0)
		{
			$sql = "SELECT user_id,
						   user_avatar,
						   user_login,
						   user_nickname,
						   user_lastvstdt
					  FROM {users}
					 WHERE user_id != 2
					   AND user_status = '1'
					   AND (TO_SECONDS(NOW()) - TO_SECONDS(user_lastvstdt)) < $s";
			if (DB::execute($sql))
			{
				$users_online = [];
				foreach (DB::fetchAll() as &$i)
				{
					$id = (int) $i['user_id'];
					$nickname = User::getNickname($i['user_login'], $i['user_nickname']);
					$users_online[$nickname] =
					[
						'avatar_source' => Avatar::getURL($id, (bool) $i['user_avatar']),
						'id' => $id,
						'last_visite' => sprintf(__('Dernière visite à %s'), 
							mb_strtolower(L10N::dt(__('%H:%M'), $i['user_lastvstdt']))
						),
						'link' => App::getURL('user/' . $id),
						'nickname' => $nickname
					];
				}
				uksort($users_online, 'Utility::alphaSort');
				Template::set('users_online', array_values($users_online));
			}
		}

		// Captures vidéos.
		if (!isset($_POST['upload'])
		&& (Auth::$isAdmin || (Config::$params['users'] && Auth::$groupPerms['upload'])))
		{
			Template::set('video_captures', Video::captures());
		}

		// Traitement de la section courante.
		require_once(__DIR__ . '/../' . str_replace('-', '_', $_GET['section_file']) . '.php');

		// Captures vidéos.
		if (isset($_POST['upload'])
		&& (Auth::$isAdmin || (Config::$params['users'] && Auth::$groupPerms['upload'])))
		{
			Template::set('video_captures', Video::captures());
		}

		// Moteur de recherche.
		Template::set('search', Config::$params['search'] ? [1] : []);
		Template::set('search_advanced', (int) Config::$params['search_advanced']);

		// Diaporama.
		Template::set('diaporama', 0);
		if (Config::$params['diaporama'])
		{
			self::_diaporama();
		}

		// Favoris.
		Template::set('favorites', Config::$params['users'] && Template::$data['user']['auth']);

		// Paramètres de thème.
		$theme_params = [];
		$p = Config::$params['theme_params'];
		if (is_array($p))
		{
			$theme_params = $p[Config::$params['theme_template']] ?? [];
		}
		Template::set('theme', ['params' => $theme_params]);

		// Requêtes quotidienne.
		self::_dailyUpdate();

		// A exécuter à la fin.
		self::makeCategoriesList();
	}

	/**
	 * Chargement du fichier de template pour les erreurs.
	 *
	 * @return mixed
	 */
	public static function error()
	{
		Template::set('page_title', function()
		{
			return __('Une erreur s\'est produite') . ' - '
				. Config::$params['gallery_title'];
		});

		include(__DIR__ . '/../error.php');

		return FALSE;
	}

	/**
	 * Génère un lien avec les paramètres de filtres.
	 *
	 * @param string $url
	 * @param bool $cat_id
	 *
	 * @return string
	 */
	public static function getURLFilter(string $url, bool $cat_id = FALSE): string
	{
		if (isset($_GET['filter']))
		{
			// Nom du filtre.
			$url .= '/' . $_GET['filter'];

			// Valeur du filtre.
			$url .= empty($_GET['filter_value'])
			|| (!$cat_id && $_GET['filter_type'] == 'filter')
				? ''
				: '/' . $_GET['filter_value'];

			// Catégorie dans laquelle s'applique le filtre.
			if ($cat_id && ($_GET['section'] != 'item'
			|| ($_GET['section'] == 'item' && isset($_GET['section_3']))))
			{
				$url .= '/' . $_GET['filter_cat_id'];
			}
		}

		return App::getURL($url);
	}

	/**
	 * Construit la liste de toutes les catégories de la galerie.
	 *
	 * @return void
	 */
	public static function makeCategoriesList(): void
	{
		if (!Config::$params['browse'] || isset(Template::$data['categories_browse']))
		{
			return;
		}

		if (!Auth::$connected && Config::$params['users_only_members'])
		{
			Template::set('categories_browse', NULL);
			return;
		}

		$get_url = function($page)
		{
			return self::getURLFilter($page);
		};
		$perms = SQL::catPerms() . " AND cat_status = '1' AND thumb_id != -1";
		$password = SQL::catPassword('select');
		$cat_id = (int) (Template::$data['category']['id'] ?? 0);
		self::$categories = Category::getList($makelist, $get_url, $cat_id, $perms, $password);

		// On supprime les sous-catégories des catégories protégées
		// par mot de passe pour lesquelles l'utilisateur n'a pas
		// l'autorisation d'y accéder.
		foreach (self::$categories as &$i)
		{
			if ($i['p_cat_id'] != $i['cat_id'] && !$i['password_auth'])
			{
				foreach (self::$categories as $id => &$i2)
				{
					if (preg_match('`\D' . $i['p_cat_id'] . '\D`', $i2['cat_parents'])
					&& $i2['p_cat_id'] != $i2['cat_id'])
					{
						unset(self::$categories[$id]);
					}
				}
			}
		}

		// Réduction de la liste.
		$reduce = function() use (&$makelist)
		{
			if (!(($_GET['filter'] == 'search' && $where = self::$search->sql())
			|| $where = SQL::itemsWhere($_GET['filter'], $_GET['filter_value'])))
			{
				return;
			}

			$sql_from = SQL::itemsFrom($_GET['filter'] ?? '');
			$sql = "SELECT cat.cat_id
					  FROM {items} AS i
						   $sql_from
				 LEFT JOIN {categories} AS cat
						ON i.cat_id = cat.cat_id
					 WHERE %s
					   AND %s
					   AND " . $where['sql'];
			$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
			if (!DB::execute($sql, $where['params']))
			{
				return;
			}
			$parents_id = DB::fetchCol('cat_id');

			Category::reduceList('album', self::$categories, $parents_id);
			Template::set('categories_browse', $makelist(self::$categories));
		};
		if (in_array($_GET['section'], ['album', 'category', 'item'])
		&& isset($_GET['filter']) && $makelist)
		{
			$reduce();
		}
		else
		{
			Template::set('categories_browse', $makelist ? $makelist(self::$categories) : NULL);
		}
	}

	/**
	 * Chargement du fichier de template pour les pages non trouvées.
	 *
	 * @return bool
	 */
	public static function notFound(): bool
	{
		Template::set('page_title', function()
		{
			return __('Page non trouvée') . ' - '
				. Config::$params['gallery_title'];
		});

		include(__DIR__ . '/../404.php');

		return FALSE;
	}



	/**
	 * Opérations à effectuer une fois par jour.
	 *
	 * @return void
	 */
	private static function _dailyUpdate(): void
	{
		if (Config::$params['daily_update'] == date('Y-m-d'))
		{
			return;
		}

		// Suppression des sessions périmées.
		DB::execute("DELETE
					   FROM {sessions}
					  WHERE session_expire < NOW()");

		// Suppression des utilisateurs en attente dont
		// la date de validation par courriel a expirée.
		DB::execute("DELETE
					   FROM {users}
					  WHERE user_status = '-2'
					    AND user_crtdt < DATE_ADD(NOW(), INTERVAL -2 DAY)");

		// Suppression des recherches périmées.
		DB::execute("DELETE
					   FROM {search}
					  WHERE search_date < DATE_ADD(NOW(), INTERVAL -2 DAY)");

		// Mise à jour de la date de dernière mise à jour.
		DB::execute("UPDATE {config}
						SET conf_value = ?
					  WHERE conf_name = 'daily_update'", date('Y-m-d'));

		// Suppression des logs utilisateurs.
		if (Config::$params['users_log_activity_delete'])
		{
			$days = (int) Config::$params['users_log_activity_delete_days'];
			DB::execute("DELETE
						   FROM {users_logs}
					      WHERE log_date < DATE_ADD(NOW(), INTERVAL -$days DAY)");
		}

		// Sauvegarde de la base de données SQLite.
		if (CONF_DB_TYPE == 'sqlite')
		{
			$db_backup_dir = GALLERY_ROOT . '/db/backup';
			$db_backup_file = $db_backup_dir . '/sqlite_' . date('Y-m-d') . '.db';
			if (file_exists(CONF_DB_NAME) && !file_exists($db_backup_file))
			{
				File::copy(CONF_DB_NAME, $db_backup_file);
			}

			// On ne garde que les 10 dernières sauvegardes.
			$n = 0;
			foreach (scandir($db_backup_dir, SCANDIR_SORT_DESCENDING) as &$f)
			{
				if (preg_match('`^sqlite_`', $f) && ++$n > 10)
				{
					File::unlink($db_backup_dir . '/' . $f);
				}
			}
		}

		// Suppression des répertoires temporaires datant de plus d'une semaine.
		$temp_dir = GALLERY_ROOT . '/cache/temp';
		if (is_dir($temp_dir))
		{
			foreach (scandir($temp_dir) as &$f)
			{
				if (is_dir($temp_dir . '/' . $f) && Utility::isSha1($f)
				&& filemtime($temp_dir . '/' . $f) < time() - 604800)
				{
					File::rmdir($temp_dir . '/' . $f);
				}
			}
		}
	}

	/**
	 * Paramètres du diaporama.
	 *
	 * @return void
	 */
	private static function _diaporama(): void
	{
		if (isset(Template::$data['category'])
		&& (in_array($_GET['section'], ['album', 'item'])
		|| ($_GET['section'] == 'category' && isset($_GET['filter']))))
		{
			$query = $_GET['q_pageless'];
			if ($_GET['section'] == 'item')
			{
				if (isset($_GET['filter']))
				{
					if ($_GET['filter_cat_id'] == '1')
					{
						$query = 'category/1';
					}
					else
					{
						foreach (Template::$data['breadcrumb'] as &$i)
						{
							if (in_array($i['type'], ['album', 'category'])
							&& $i['id'] == $_GET['filter_cat_id'])
							{
								$query = $i['type'] . '/' . $i['id'];
								break;
							}
						}
					}
					$query .= '/' . $_GET['filter'];
					if ($_GET['filter_type'] == 'filter_p')
					{
						$query .= '/' . $_GET['filter_value'];
					}
				}
				else
				{
					$query = Template::$data['category']['type'] . '/'
						. Template::$data['category']['id'];
				}
				$position = Template::$data['item_position'];
			}
			else
			{
				$position = (($_GET['page'] - 1)
					* (int) Config::$params['thumbs_item_nb_per_page']) + 1;
			}

			$settings = [];

			Template::set('diaporama', 1);
			Template::set('diaporama_position', (int) $position);
			Template::set('diaporama_query', $query);
			Template::set('diaporama_settings',
			[
				'autoDuration' => (int) Config::$params['diaporama_auto_duration'],
				'autoLoop' => Config::$params['diaporama_auto_loop']
					? 'true' : 'false',
				'autoStart' => Config::$params['diaporama_auto_start']
					? 'true' : 'false',
				'carousel' => Config::$params['diaporama_carousel']
					? 'true' : 'false',
				'carouselThumbsSize' => (int) Config::$params['diaporama_carousel_thumbs_size'],
				'fullScreen' => Config::$params['diaporama_fullscreen']
					? 'true' : 'false',
				'fullScreenMobile' => Config::$params['diaporama_fullscreen_mobile']
					? 'true' : 'false',
				'hideControlBars' => Config::$params['diaporama_hide_control_bars']
					? 'true' : 'false',
				'transitionDuration' => (int) Config::$params['diaporama_transition_duration'],
				'transitionEffect' => Config::$params['diaporama_transition_effect']
			]);
		}
	}

	/**
	 * Génère les informations de template pour la pagination.
	 *
	 * @param mixed $count
	 *   Nombre de pages.
	 *
	 * @return void
	 */
	protected static function _pagination($count): void
	{
		if (!$q = $_GET['q_pageless'])
		{
			$q = 'category/1-' . __('galerie');
		}
		Template::set('pages',
		[
			'count' => (int) $count,
			'first' =>
			[
				'active' => $_GET['page'] > 1,
				'link' => App::getURL($q . '/page/1')
			],
			'prev' =>
			[
				'active' => $_GET['page'] > 1,
				'link' => App::getURL($q . '/page/'
					. ($_GET['page'] > 1 ? ($_GET['page'] - 1) : 1))
			],
			'next' =>
			[
				'active' => $count > $_GET['page'],
				'link' => App::getURL($q . '/page/'
					. ($count > 1 ? ($_GET['page'] + 1) : 1))
			],
			'last' =>
			[
				'active' => $count > $_GET['page'],
				'link' => App::getURL($q . '/page/' . (int) $count)
			]
		]);
	}

	/**
	 * Gestion des paramètres GET.
	 *
	 * @return void
	 */
	private static function _request(): void
	{
		$q = !empty($_GET['q']);

		// Configuration de base.
		$request =
		[
			// Catégories.
			'{category}/{id}',
			'{category}/{id}/camera-brand/{id}',
			'{category}/{id}/camera-model/{id}',
			'{category}/{id}/date-added/{date-flex}',
			'{category}/{id}/date-created/{date-flex}',
			'{category}/{id}/images',
			'{category}/{id}/items',
			'{category}/{id}/videos',
			'{category}/{id}/views',

			// Fichiers.
			'item/{id}',
			'item/{id}/camera-brand/{id}/{id}',
			'item/{id}/camera-model/{id}/{id}',
			'item/{id}/date-added/{date-flex}/{id}',
			'item/{id}/date-created/{date-flex}/{id}',
			'item/{id}/images/{id}',
			'item/{id}/items/{id}',
			'item/{id}/videos/{id}',
			'item/{id}/views/{id}',

			// Mot de passe de catégorie.
			'logout/{id}',
			'password/{category}/{id}',
			'password/item/{id}',

			// Pages personnalisées.
			'page/{id}'
		];

		// Page des appareils photos.
		if (Config::$params['pages_params']['cameras']['status'])
		{
			$request[] = 'cameras';
			$request[] = 'cameras/{id}';
		}

		// Commentaires.
		if (Config::$params['comments'])
		{
			// Page des commentaires.
			if (Config::$params['pages_params']['comments']['status'])
			{
				$request[] = 'comments';
				$request[] = 'comments/{category}/{id}';
				$request[] = 'comments/{category}/{id}/user/{id}';
				$request[] = 'comments/user/{id}';
			}

			// Filtre.
			$request[] = '{category}/{id}/comments';
			$request[] = 'item/{id}/comments/{id}';
		}

		// Page contact.
		if (Config::$params['pages_params']['contact']['status'])
		{
			$request[] = 'contact';
			$request[] = 'contact/confirm';
		}

		// Fichiers récents.
		if (Config::$params['items_recent'])
		{
			$request[] = '{category}/{id}/recent-images';
			$request[] = '{category}/{id}/recent-items';
			$request[] = '{category}/{id}/recent-videos';
			$request[] = 'item/{id}/recent-images/{id}';
			$request[] = 'item/{id}/recent-items/{id}';
			$request[] = 'item/{id}/recent-videos/{id}';
		}

		// Langues.
		if (Config::$params['lang_switch'])
		{
			$request[] = 'language';
		}

		// Moteur de recherche.
		if (Config::$params['search'])
		{
			$request[] = 'search';

			// Recherche avancée.
			if (Config::$params['search_advanced'])
			{
				$request[] = 'search-advanced';
			}

			// Filtre.
			$request[] = '{category}/{id}/search/{search}';
			$request[] = 'item/{id}/search/{search}/{id}';
		}

		// Tags.
		if (Config::$params['tags'])
		{
			// Page des tags.
			$request[] = 'tags';
			$request[] = 'tags/{id}';

			// Filtre.
			$request[] = '{category}/{id}/tag/{id}';
			$request[] = 'item/{id}/tag/{id}/{id}';
		}

		// Utilisateurs.
		if (Config::$params['users'])
		{
			// Connexion / déconnexion.
			$request[] = 'login';
			$request[] = 'logout';

			// Inscription.
			if (Config::$params['users_registration'])
			{
				$request[] = 'register';
				$request[] = 'validation/{key}';
			}

			// Pages pour la procédure d'oubli du mot de passe.
			$request[] = 'forgot-password';
			$request[] = 'new-password';

			// Page de profil des membres.
			$request[] = 'user/{id}';

			// Pages de modifications du profil.
			$request[] = 'user-avatar';
			$request[] = 'user-options';
			$request[] = 'user-password';
			$request[] = 'user-profile';

			// Page d'ajout de fichiers.
			$request[] = 'user-upload';
			$request[] = 'user-upload/album/{id}';

			// Liste des membres.
			if (Config::$params['pages_params']['members']['status'])
			{
				$request[] = 'members';
				$request[] = 'members/group/{id}';
			}

			// Filtres.
			$request[] = '{category}/{id}/user-favorites/{id}';
			$request[] = '{category}/{id}/user-images/{id}';
			$request[] = '{category}/{id}/user-items/{id}';
			$request[] = '{category}/{id}/user-videos/{id}';
			$request[] = '{category}/{id}/favorites';
			$request[] = 'item/{id}/user-favorites/{id}/{id}';
			$request[] = 'item/{id}/user-images/{id}/{id}';
			$request[] = 'item/{id}/user-items/{id}/{id}';
			$request[] = 'item/{id}/user-videos/{id}/{id}';
			$request[] = 'item/{id}/favorites/{id}';
		}

		// Votes.
		if (Config::$params['votes'])
		{
			// Filtre.
			$request[] = '{category}/{id}/votes';
			$request[] = 'item/{id}/votes/{id}';
		}

		// Géolocalisation.
		if (Config::$params['geolocation'])
		{
			// Page de la carte du monde.
			if (Config::$params['pages_params']['worldmap']['status'])
			{
				$request[] = 'worldmap';
			}
		}

		// Analyse de la requête.
		App::request($request);

		// Paramètres GET par défaut.
		if (!isset($_GET['section']))
		{
			$_GET['q'] = $_GET['q_pageless'] = $_GET['section'] = $q ? '404' : 'category';
		}
		if (isset($_GET['album_id']))
		{
			$_GET['category_id'] = $_GET['album_id'];
		}
		if (!isset($_GET['category_id']))
		{
			$_GET['category_id'] = 1;
		}
		if ($_GET['q_pageless'] == 'category' && $_GET['category_id'] == 1)
		{
			$_GET['q_pageless'] = '';
		}
		$_GET['section_file'] = $_GET['section'];
	}

	/**
	 * Gestion du moteur de recherche.
	 *
	 * @return void
	 */
	private static function _search(): void
	{
		if (isset($_GET['search']))
		{
			self::$search = new Search();
			if ($search = self::$search->sql($_GET['search'])
			&& !isset(self::$search->options['type']))
			{
				Template::set('search', ['query' => self::$search->query]);
			}
			else
			{
				Template::set('search', ['invalid' => 1]);
			}
		}

		// Données POST.
		if (isset($_POST['search_query']) && mb_strlen($_POST['search_query']) <= 68
		&& !Utility::isEmpty($_POST['search_query']))
		{
			self::$search = new Search();

			// Options de recherche.
			if (!Config::$params['search_advanced'] || empty($_POST['search_options']))
			{
				$options = ['columns' => ['item_name', 'item_desc',], 'item_tags' => 1];
			}
			else
			{
				// Vérification des options.
				$check = $_POST['search_options'];
				$options = [];

				// Tous les mots ?
				if (isset($check['all_words']))
				{
					$options['all_words'] = 1;
				}

				// Champs de recherche.
				if (!isset($check['columns']) || !is_array($check['columns']))
				{
					return;
				}
				$options['columns'] = [];
				foreach ($check['columns'] as &$col)
				{
					if (in_array($col, ['desc', 'name']))
					{
						$options['columns'][] = 'item_' . $col;
					}
					else if (in_array($col, ['comments', 'tags']))
					{
						$options['item_' . $col] = 1;
					}
				}

				// Date.
				if (!empty($check['date']) && isset($check['date_column'])
				&& in_array($check['date_column'], ['adddt', 'crtdt']))
				{
					$options['date'] = 1;
					$options['date_column'] = 'item_' . $check['date_column'];
					foreach (['start', 'end'] as $p1)
					{
						foreach (['day', 'month', 'year'] as $p2)
						{
							if (!isset($check['date_' . $p1 . '_' . $p2])
							|| !preg_match('`^\d{2,4}$`', $check['date_' . $p1 . '_' . $p2]))
							{
								unset($options['date']);
								break;
							}
							$options['date_' . $p1 . '_' . $p2]
								= $check['date_' . $p1 . '_' . $p2];
						}
					}
				}

				// Dimensions.
				if (!empty($check['size']))
				{
					$options['item_size'] = 1;
					foreach (['width', 'height'] as $p1)
					{
						foreach (['start', 'end'] as $p2)
						{
							if (!isset($check['size_' . $p1 . '_' . $p2])
							|| !preg_match('`^\d{1,6}$`', $check['size_' . $p1 . '_' . $p2]))
							{
								unset($options['item_size']);
								break;
							}
							$options['item_size_' . $p1 . '_' . $p2]
								= $check['size_' . $p1 . '_' . $p2];
						}
					}
				}

				// Poids.
				if (!empty($check['filesize']))
				{
					$options['item_filesize'] = 1;
					foreach (['start', 'end'] as $p1)
					{
						if (!isset($check['filesize_' . $p1])
						|| !preg_match('`^\d{1,8}$`', $check['filesize_' . $p1]))
						{
							unset($options['item_filesize']);
							break;
						}
						$options['item_filesize_' . $p1] = $check['filesize_' . $p1];
					}
				}
			}

			// Enregistrement des paramètres de la recherche et redirection.
			if ($search_id = self::$search->insert($_POST['search_query'], $options))
			{
				App::redirect('category/1/search/' . $search_id);
			}
		}
	}
}
?>