<?php
declare(strict_types = 1);

require_once(__DIR__ . '/Gallery.class.php');

/**
 * Opérations concernant les catégories.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class GalleryCategory extends Gallery
{
	/**
	 * Informations en base de données de la catégorie courante.
	 *
	 * @var array
	 */
	public static $infos = [];



	/**
	 * Informations en base de données des catégories.
	 *
	 * @var array
	 */
	private static $_categories = [];

	/**
	 * Nombre de pages de la catégorie courante.
	 *
	 * @var int
	 */
	private static $_pagesCount = 0;



	/**
	 * Vérification du mot de passe entré par l'utilisateur
	 * pour l'accès à une partie protégée de la galerie.
	 *
	 * @param string $type
	 * @param int $object_id
	 * @param string $password
	 * @param bool $remember
	 *
	 * @return string
	 */
	public static function checkPassword(string $type, int $object_id,
	string $password, bool $remember): string
	{
		// Récupération des informations de l'objet demandé.
		if ($type == 'item')
		{
			$sql = "SELECT i.item_id AS id,
						   i.item_url AS url,
						   p.cat_id AS p_cat_id,
						   p.password_hash,
						   'item' AS type,
						   %s
					  FROM {items} AS i
				 LEFT JOIN {categories} AS cat USING(cat_id)
			     LEFT JOIN {passwords} AS p USING (password_id)
					 WHERE %s
					   AND i.item_id = ?";
		}
		else
		{
			$sql = "SELECT cat.cat_id AS id,
						   cat.cat_url AS url,
						   p.cat_id AS p_cat_id,
						   p.password_hash,
						   CASE WHEN cat_filemtime IS NULL
								THEN 'category'
								ELSE 'album'
								 END AS type,
						   %s
					  FROM {categories} AS cat
			     LEFT JOIN {passwords} AS p USING (password_id)
					 WHERE %s
					   AND cat.cat_id = ?";
		}
		$sql = sprintf($sql, SQL::catPassword('select'), SQL::catPerms());
		if (!DB::execute($sql, $object_id))
		{
			return 'db_error';
		}
		$i = DB::fetchRow();
		if (!array_key_exists('password_auth', $i))
		{
			return 'not_found';
		}

		// Si l'objet n'est pas protégé par un mot de passe,
		// ou si l'utilisateur a déjà entré le bon mot de passe,
		// on redirige vers la page de l'objet.
		if ($i['password_auth'])
		{
			return 'redirect:' . $i['type'] . '/' . $i['id'] . '-' . $i['url'];
		}

		// On arrête là si l'utilisateur n'a pas envoyé de mot de passe.
		if (empty($password))
		{
			return 'no_password';
		}

		// Prévention contre les attaques par force brute.
		if (Auth::isBruteForce('password\_rejected'))
		{
			return 'bruteforce';
		}

		// Vérification du mot de passe entré par l'utilisateur.
		if (!App::passwordVerify($password, $i['password_hash']))
		{
			// Log d'activité.
			App::logActivity('password_rejected');

			return 'password_error';
		}

		// Début de la transaction.
		if (!DB::beginTransaction())
		{
			return 'db_error';
		}

		// On génère un nouveau jeton de session.
		$session = Auth::getNewSessionToken();
		if (!$session['id'])
		{
			return 'db_error';
		}

		// On associe l'identifiant de session avec la catégorie.
		$sql = "INSERT INTO {sessions_categories} (session_id, cat_id) VALUES (?, ?)";
		$params = [$session['id'], $i['p_cat_id']];
		if (!DB::execute($sql, $params))
		{
			return 'db_error';
		}

		// Log d'activité.
		App::logActivity('password');

		// Exécution de la transaction.
		if (!DB::commitTransaction())
		{
			return 'db_error';
		}

		// Mise à jour du cookie de session.
		Auth::$session->add('session_token', $session['token']);
		Auth::$session->add(
			'session_expire', $remember ? (string) CONF_COOKIE_SESSION_EXPIRE : '0'
		);

		// Redirection vers la page demandée.
		return 'redirect:' . $i['type'] . '/' . $i['id'] . '-' . $i['url'];
	}

	/**
	 * Récupération des catégories de la page courante.
	 *
	 * @return int
	 */
	public static function getCategories(): int
	{
		self::_pagination(1);

		// Nombre de catégories par page.
		$cats_per_page = (int) Config::$params['thumbs_cat_nb_per_page'];
		Template::set('cats_per_page', $cats_per_page);

		// Clause WHERE.
		$sql_where = "%s
				  AND cat.cat_id > 1
				  AND cat_status = '1'
				  AND thumb_id > 0
				  AND cat_a_images + cat_a_videos > 0
				  AND cat.parent_id = " . (int) $_GET['category_id'];

		// Clause LIMIT.
		$sql_limit = $cats_per_page * ($_GET['page'] - 1) . ",$cats_per_page";

		// Clause ORDER BY.
		$sql_order_by = SQL::catOrderBy(
			self::$infos['cat_orderby'] ?? Config::$params['categories_sql_order_by'],
			self::$infos
		);

		// Récupération des catégories.
		$sql = "SELECT cat.*,
					   %s,
					   CASE WHEN cat_filemtime IS NULL
							THEN 'category'
							ELSE 'album'
							 END AS cat_type,
					   i.item_id,
					   i.item_type,
					   i.item_path,
					   i.item_width,
					   i.item_height,
					   i.item_adddt,
					   u.user_login,
					   u.user_status
				  FROM {categories} AS cat
			 LEFT JOIN {users} AS u
					ON cat.user_id = u.user_id
			 LEFT JOIN {items} AS i
					ON cat.thumb_id = i.item_id
				 WHERE $sql_where
			  ORDER BY $sql_order_by
			     LIMIT $sql_limit";
		$sql = sprintf($sql, SQL::catPassword('select'), SQL::catPerms());
		if (!DB::execute($sql))
		{
			return -1;
		}
		if (!self::$_categories = DB::fetchAll('cat_id'))
		{
			if ($_GET['page'] > 1)
			{
				App::redirect($_GET['q_pageless']);
				return 1;
			}
			else
			{
				return (int) ($_GET['category_id'] == 1);
			}
		}

		// Réduction des stats.
		if (!self::changeCatStats(self::$_categories))
		{
			return -1;
		}

		// Nombre de catégories de la section courante.
		$sql = "SELECT COUNT(*) FROM {categories} AS cat WHERE $sql_where";
		if (!DB::execute(sprintf($sql, SQL::catPerms())))
		{
			return -1;
		}
		Template::set('objects_count', DB::fetchVal());

		// Nombre de pages.
		self::$_pagesCount = ceil(Template::$data['objects_count'] / $cats_per_page);
		self::_pagination(self::$_pagesCount);

		// Formatage des informations.
		foreach (self::$_categories as &$i)
		{
			$formated_infos[$i['cat_id']] = self::getFormatedInfos($i, 'thumb');
		}
		Template::set('categories', $formated_infos);

		return 1;
	}

	/**
	 * Formate les informations d'une catégorie.
	 *
	 * @param array $i
	 *   Informations brutes de la catégorie.
	 * @param string $origin
	 *   Origine : 'category' ou 'thumb'.
	 *
	 * @return array
	 */
	public static function getFormatedInfos(array &$i, string $origin = 'category'): array
	{
		$items = $i['cat_a_images'] + $i['cat_a_videos'];
		$link = ($i['password_auth'])
			? $i['cat_type'] . '/' . $i['cat_id'] . '-' . $i['cat_url']
			: 'password/' . $i['cat_type'] . '/' . $i['cat_id'];
		$filter_link = App::getURL($i['cat_type'] . '/' . $i['cat_id']);

		// Uploadable ?
		$uploadable = FALSE;
		if (!empty($i['parent_uploadable']) && $i['cat_uploadable']
		&& $i['cat_type'] == 'album' && Config::$params['users']
		&& Auth::$groupPerms['upload']
		&& (!Auth::$groupPerms['upload_owner']
		 || (Auth::$groupPerms['upload_owner']
			&& $i['user_id'] == Auth::$infos['user_id'])))
		{
			$uploadable = TRUE;
		}

		// Statistiques.
		$stats_order = Config::$params['categories_stats_order'];
		if ($origin == 'category' && Config::$params['categories_stats']
		&& in_array($_GET['section'], ['album', 'category']))
		{
			$stats_params = Config::$params['categories_stats_params'];
		}
		$stats = [];
		foreach ($stats_order as &$name)
		{
			if (($origin == 'category' && isset($stats_params) && $stats_params[$name]['status'])
			|| $origin == 'thumb')
			{
				$stats[$name] = [];
				switch ($name)
				{
					case 'albums' :
						if ($i['cat_type'] == 'category' && ($origin == 'category'
						|| Config::$params['thumbs_cat_info_albums']))
						{
							$stats[$name] =
							[
								'text' => $i['cat_a_albums'] > 1
									? sprintf(__('%s albums'),
										L10N::formatNumber((int) $i['cat_a_albums']))
									: sprintf(__('%s album'),
										L10N::formatNumber((int) $i['cat_a_albums'])),
								'link' => NULL,
								'short' => L10N::formatShortNumber((int) $i['cat_a_albums']),
								'value' => $i['cat_a_albums']
							];
						}
						else if ($origin == 'category')
						{
							unset($stats[$name]);
						}
						break;

					case 'comments' :
						if (Config::$params['comments'] && ($origin == 'category'
						|| Config::$params['thumbs_cat_info_comments']))
						{
							$stats[$name] =
							[
								'text' => $i['cat_a_comments'] > 1
									? sprintf(__('%s commentaires'),
										L10N::formatNumber((int) $i['cat_a_comments']))
									: sprintf(__('%s commentaire'),
										L10N::formatNumber((int) $i['cat_a_comments'])),
								'link' => $i['cat_a_comments'] > 0
									? $filter_link . '/comments'
									: NULL,
								'short' => L10N::formatShortNumber((int) $i['cat_a_comments']),
								'value' => $i['cat_a_comments']
							];
						}
						else if ($origin == 'category')
						{
							unset($stats[$name]);
						}
						break;

					case 'favorites' :
						if (Config::$params['users'] && ($origin == 'category'
						|| Config::$params['thumbs_cat_info_favorites']))
						{
							$stats[$name] =
							[
								'text' => $i['cat_a_favorites'] > 1
									? sprintf(__('%s favoris'),
										L10N::formatNumber((int) $i['cat_a_favorites']))
									: sprintf(__('%s favori'),
										L10N::formatNumber((int) $i['cat_a_favorites'])),
								'link' => $i['cat_a_favorites'] > 0
									? $filter_link . '/favorites'
									: NULL,
								'short' => L10N::formatShortNumber((int) $i['cat_a_favorites']),
								'value' => $i['cat_a_favorites']
							];
						}
						else if ($origin == 'category')
						{
							unset($stats[$name]);
						}
						break;

					case 'images' :
						if ($origin == 'category' || Config::$params['thumbs_cat_info_items'])
						{
							$stats[$name] =
							[
								'text' => $i['cat_a_images'] > 1
									? sprintf(__('%s photos'),
										L10N::formatNumber((int) $i['cat_a_images']))
									: sprintf(__('%s photo'),
										L10N::formatNumber((int) $i['cat_a_images'])),
								'link' => $i['cat_a_images'] > 0
									? $filter_link . '/images'
									: NULL,
								'short' => L10N::formatShortNumber((int) $i['cat_a_images']),
								'value' => $i['cat_a_images']
							];
						}
						break;

					case 'items' :
						$stats[$name] =
						[
							'text' => $items > 1
								? sprintf(__('%s photos et vidéos'),
									L10N::formatNumber((int) $items))
								: sprintf(__('%s photo ou vidéo'),
									L10N::formatNumber((int) $items)),
							'link' => $items > 0 ? $filter_link . '/items' : NULL,
							'short' => L10N::formatShortNumber((int) $items),
							'value' => $items
						];
						break;

					case 'filesize' :
						if ($origin == 'category' || Config::$params['thumbs_cat_info_filesize'])
						{
							$stats[$name] =
							[
								'text' => L10N::formatFilesize((int) $i['cat_a_size']),
								'link' => NULL,
								'value' => (int) $i['cat_a_size']
							];
						}
						break;

					case 'rating' :
						if (Config::$params['votes'] && ($origin == 'category'
						|| Config::$params['thumbs_cat_info_rating']))
						{
							$stats['rating'] =
							[
								'array' => Rating::formatArray($i['cat_a_rating'] ?? 0),
								'text' => L10N::formatRating($i['cat_a_rating'] ?? 0),
								'link' => NULL,
								'value' => $i['cat_a_rating']
							];
						}
						else if ($origin == 'category')
						{
							unset($stats[$name]);
						}
						break;

					case 'recent_images' :
						$stats[$name] =
						[
							'text' => sprintf(__('%s nouvelle photo'), 0),
							'link' => NULL,
							'value' => 0
						];
						break;

					case 'recent_items' :
						$stats[$name] =
						[
							'text' => sprintf(__('%s nouvelle photo ou vidéo'), 0),
							'link' => NULL,
							'value' => 0
						];
						break;

					case 'recent_videos' :
						$stats[$name] =
						[
							'text' => sprintf(__('%s nouvelle vidéo'), 0),
							'link' => NULL,
							'value' => 0
						];
						break;

					case 'videos' :
						if ($origin == 'category' || Config::$params['thumbs_cat_info_items'])
						{
							$stats[$name] =
							[
								'text' => $i['cat_a_videos'] > 1
									? sprintf(__('%s vidéos'),
										L10N::formatNumber((int) $i['cat_a_videos']))
									: sprintf(__('%s vidéo'),
										L10N::formatNumber((int) $i['cat_a_videos'])),
								'link' => $i['cat_a_videos'] > 0 ? $filter_link . '/videos' : 0,
								'short' => L10N::formatShortNumber((int) $i['cat_a_videos']),
								'value' => $i['cat_a_videos']
							];
						}
						break;

					case 'views' :
						if ($origin == 'category' || Config::$params['thumbs_cat_info_views'])
						{
							$stats[$name] =
							[
								'text' => $i['cat_a_hits'] > 1
									? sprintf(__('%s vues'),
										L10N::formatNumber((int) $i['cat_a_hits']))
									: sprintf(__('%s vue'),
										L10N::formatNumber((int) $i['cat_a_hits'])),
								'link' => $i['cat_a_hits'] > 0 ? $filter_link . '/views' : NULL,
								'short' => L10N::formatShortNumber((int) $i['cat_a_hits']),
								'value' => $i['cat_a_hits']
							];
						}
						break;

					case 'votes' :
						if (Config::$params['votes'] && ($origin == 'category'
						|| Config::$params['thumbs_cat_info_votes']))
						{
							$stats[$name] =
							[
								'text' => $i['cat_a_votes'] > 1
									? sprintf(__('%s votes'),
										L10N::formatNumber((int) $i['cat_a_votes']))
									: sprintf(__('%s vote'),
										L10N::formatNumber((int) $i['cat_a_votes'])),
								'link' => $i['cat_a_votes'] > 0 ? $filter_link . '/votes' : NULL,
								'short' => L10N::formatShortNumber((int) $i['cat_a_votes']),
								'value' => $i['cat_a_votes']
							];
						}
						else if ($origin == 'category')
						{
							unset($stats[$name]);
						}
				}
			}
		}

		// Fichiers récents.
		$recent = ['value' => 0, 'link' => NULL];
		$cat_type_text = $i['cat_type'] == 'category' ? __('cette catégorie') : __('cet album');
		if (!$i['cat_a_videos'] && $i['cat_a_images'])
		{
			$recent['text'] = sprintf(
				__('Montrer les photos récentes de %s'),
				$cat_type_text
			);
			$recent['link'] = App::getURL($i['cat_type'] . '/' . $i['cat_id'] . '/recent-images');
		}
		if ($i['cat_a_videos'] && !$i['cat_a_images'])
		{
			$recent['text'] = sprintf(
				__('Montrer les vidéos récentes de %s'),
				$cat_type_text
			);
			$recent['link'] = App::getURL($i['cat_type'] . '/' . $i['cat_id']) . '/recent-videos';
		}
		if ($i['cat_a_videos'] && $i['cat_a_images'])
		{
			$recent['text'] = sprintf(
				__('Montrer les photos et vidéos récentes de %s'),
				$cat_type_text
			);
			$recent['link'] = App::getURL($i['cat_type'] . '/' . $i['cat_id'] . '/recent-items');
		}

		// Description.
		$description = isset($_GET['filter']) ? NULL : ($i['cat_id'] > 1
			? $i['cat_desc']
			: Config::$params['gallery_description']);

		// Téléchargement de l'album.
		$download_link = '';
		if ($origin == 'category' && $i['cat_type'] == 'album' && $i['cat_downloadable']
		&& !isset($_GET['filter']) && Config::$params['albums_download']
		&& (!Config::$params['users']
		|| (Config::$params['users'] && Auth::$groupPerms['albums_download'])))
		{
			$key = App::fileKeyHash([$i['cat_id']]);
			$download_link = CONF_GALLERY_PATH . "/download.php?key=$key&album={$i['cat_id']}";
		}

		return
		[
			'id' => $i['cat_id'],
			'type' => $i['cat_type'],
			'canonical' => $i['cat_id'] == 1 ? GALLERY_HOST . App::getURL() : NULL,
			'link' => App::getURL($link),
			'link_admin' => App::getURLAdmin($i['cat_type'] . '/' . $i['cat_id']),
			'objects_count' => $items,
			'path' => $i['cat_path'],
			'download' => $download_link ? 1 : 0,
			'download_link' => $download_link,
			'upload' => $uploadable,
			'urlname' => $i['cat_url'],

			// Mot de passe.
			'link_logout' => App::getURL('logout/' . $i['cat_id']),
			'locked' => (bool) !$i['password_auth'],
			'unlocked' => $i['password_id'] && $i['password_auth'],

			// Informations.
			'description' => $description,
			'description_meta' => $origin == 'category'
				? App::getMetaDescription((string) $description)
				: '',
			'title' => ($origin == 'category' || Config::$params['thumbs_cat_info_title'])
				? $i['cat_name']
				: '',

			// Statistiques.
			'recent' => $recent,
			'stats' => $stats,

			// Vignette.
			'thumb_id' => (int) $i['item_id'],
			'thumb_size' => function(int $max_width, int $max_height) use ($i):array
			{
				return Image::getResizedSize((int) $i['item_width'],
					(int) $i['item_height'], $max_width, $max_height);
			},
			'thumb_src' => function($size = '', $quality = '') use ($i)
			{
				return htmlspecialchars(($i['thumb_id'] == -1)
					? ''
					: App::getThumbSource('cat', $i, (string) $size, (string) $quality));
			},
			'thumb_type' => Item::isVideo($i['item_type']) ? 'video' : 'image'
		];
	}

	/**
	 * Récupération des informations de la catégorie courante.
	 *
	 * @param string $type
	 * @param int $cat_id
	 *
	 * @return int
	 */
	public static function getInfos(string $type, int $cat_id): int
	{
		if ($cat_id == 1 && $type != 'category')
		{
			App::redirect();
			return 1;
		}

		// Type de catégorie.
		$sql_type = ($type == 'category')
			? " AND cat_filemtime IS NULL "
			: " AND cat_filemtime IS NOT NULL ";

		// Récupération des informations.
		$sql = "SELECT cat.*,
					   %s,
					   CASE WHEN cat_filemtime IS NULL
						    THEN 'category' ELSE 'album'
							 END AS cat_type,
					   i.item_id,
					   i.item_type
				  FROM {categories} AS cat
			 LEFT JOIN {items} AS i
					ON cat.thumb_id = i.item_id
				 WHERE %s
				   AND cat.cat_id = ?
				   AND cat_status = '1'"
					 . $sql_type;
		$sql = sprintf($sql, SQL::catPassword('select'), SQL::catPerms());
		if (!DB::execute($sql, $cat_id))
		{
			return -1;
		}
		if (!isset((self::$infos = DB::fetchRow())['cat_id']))
		{
			return 0;
		}

		// L'utilisateur a-t-il entré le bon mot de passe ?
		if (!self::checkCookiePassword(self::$infos))
		{
			return 0;
		}

		// Réduction des statistiques.
		if (!self::changeCatStats(self::$infos))
		{
			return -1;
		}

		// Réglages.
		self::$infos = [self::$infos];
		Parents::settings(self::$infos);
		self::$infos = self::$infos[0];

		// Fil d'Ariane.
		Template::breadcrumb(self::$infos, TRUE);
		unset(Template::$data['breadcrumb'][0]);

		// Numéro de page pour la catégorie parente du fil d'Ariane.
		if ($_GET['category_id'] > 1 && !isset($_GET['filter']))
		{
			$index = FALSE;
			foreach (Template::$data['breadcrumb'] as $k => &$i)
			{
				if ($i['id'] == self::$infos['parent_id'])
				{
					$index = $k;
					break;
				}
			}
			if ($index !== FALSE)
			{
				$cat_per_page = (int) Config::$params['thumbs_cat_nb_per_page'];
				$parent = Template::$data['breadcrumb'][$index];
				if (self::$infos['parent_id'] == 1 || $parent['a_cat'] > $cat_per_page)
				{
					$sql_order_by = SQL::catOrderBy(
						$parent['order_by'] ?? Config::$params['categories_sql_order_by'],
						$parent
					);
					$sql = "SELECT cat_id
							  FROM {categories} AS cat
							 WHERE %s
							   AND parent_id = ?
							   AND cat_id > 1
							   AND cat_status = '1'
							   AND thumb_id > 0
						  ORDER BY $sql_order_by";
					$sql = sprintf($sql, SQL::catPerms());
					if (DB::execute($sql, $parent['id']))
					{
						$categories = array_values(DB::fetchAll('cat_id', 'cat_id'));
						$cat_position = array_search(
							self::$infos['cat_id'],
							$categories
						) + 1;
						$parent_page = ceil($cat_position / $cat_per_page);
						if ($parent_page > 1)
						{
							Template::$data['breadcrumb'][$index]['url']
								.= '/page/' . $parent_page;
						}
					}
				}
			}
		}
		foreach (Template::$data['breadcrumb'] as &$i)
		{
			if (array_key_exists('a_cat', $i))
			{
				unset($i['a_cat']);
			}
			if (array_key_exists('order_by', $i))
			{
				unset($i['order_by']);
			}
		}

		// Formatage des informations.
		Template::set('category', self::getFormatedInfos(self::$infos, 'category'));

		// Récupération des tags.
		if (Config::$params['tags'])
		{	
			Template::set('category',
			[
				'tags' => self::getTags(self::$infos),
				'tags_link' => $cat_id > 1 ? App::getURL('tags/' . $cat_id) : App::getURL('tags')
			]);
		}

		return 1;
	}

	/**
	 * Récupère le nombre de fichiers récents de la catégorie
	 * et des sous-catégories.
	 *
	 * @return void
	 */
	public static function getRecentItemsCount(): void
	{
		// S'il ne faut pas mettre en évidence les fichiers récents
		// ou s'il n'y a aucun nouveau fichier, on arrête là.
		if (!Config::$params['items_recent'] || empty(self::$infos['cat_lastadddt'])
		|| strtotime(self::$infos['cat_lastadddt']) < Item::getRecentTimestamp())
		{
			return;
		}

		// Requête SQL pour déterminer le nombre de fichiers récents d'une catégorie.
		$get_recent_items = function(&$i)
		{
			$sql = "SELECT COUNT(*) AS count,
						   item_type
					  FROM {items}
				 LEFT JOIN {categories} AS cat USING (cat_id)
					 WHERE %s
					   AND %s
					   AND item_path LIKE ?
					   AND item_adddt > ?
					   AND item_status = '1'
				  GROUP BY item_type";
			$cat_path = $i['cat_id'] > 1
				? DB::likeEscape($i['cat_path']) . '/%'
				: '%';
			$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
			$params = [$cat_path, date('Y-m-d H:i:s', Item::getRecentTimestamp())];
			if (!DB::execute($sql, $params))
			{
				return FALSE;
			}

			// Assemblage des données.
			$recent = DB::fetchAll('item_type');
			$recent_images = 0;
			$recent_items = 0;
			$recent_videos = 0;
			foreach (Item::IMAGE_TYPES as &$type)
			{
				$recent_items += $recent[$type]['count'] ?? 0;
				$recent_images += $recent[$type]['count'] ?? 0;
			}
			foreach (Item::VIDEO_TYPES as &$type)
			{
				$recent_items += $recent[$type]['count'] ?? 0;
				$recent_videos += $recent[$type]['count'] ?? 0;
			}
			return
			[
				'images' => $recent_images,
				'items' => $recent_items,
				'videos' => $recent_videos
			];
		};

		// Formatage des données.
		$tpl = function(&$t, &$r)
		{
			$filter_link = App::getURL($t['type'] . '/' . $t['id']);

			// Images.
			if (isset($t['stats']['recent_images']))
			{
				$formated = L10N::formatNumber((int) $r['images']);
				$t['stats']['recent_images']['text'] = $r['images'] > 1
					? sprintf(__('%s nouvelles photos'), $formated)
					: sprintf(__('%s nouvelle photo'), $formated);
				$t['stats']['recent_images']['link'] = $r['images'] > 0
					? $filter_link . '/recent-images'
					: NULL;
				$t['stats']['recent_images']['value'] = $r['images'];
				$t['stats']['recent_images']['formated'] = $formated;
			}

			// Images + vidéos.
			if (isset($t['stats']['recent_items']))
			{
				$formated = L10N::formatNumber((int) $r['items']);
				$t['stats']['recent_items']['text'] = $r['items'] > 1
					? sprintf(__('%s nouvelles photos et vidéos'), $formated)
					: sprintf(__('%s nouvelle photo ou vidéo'), $formated);
				$t['stats']['recent_items']['link'] = $r['items'] > 0
					? $filter_link . '/recent-items'
					: NULL;
				$t['stats']['recent_items']['value'] = $r['items'];
				$t['stats']['recent_items']['formated'] = $formated;
			}

			// Vidéos.
			if (isset($t['stats']['recent_videos']))
			{
				$formated = L10N::formatNumber((int) $r['videos']);
				$t['stats']['recent_videos']['text'] = $r['videos'] > 1
					? sprintf(__('%s nouvelles vidéos'), $formated)
					: sprintf(__('%s nouvelle vidéo'), $formated);
				$t['stats']['recent_videos']['link'] = $r['videos'] > 0
					? $filter_link . '/recent-videos'
					: NULL;
				$t['stats']['recent_videos']['value'] = $r['videos'];
				$t['stats']['recent_videos']['formated'] = $formated;
			}

			// Fichiers récents.
			$t['recent']['value'] = $t['stats']['recent_items']['value'] ?? NULL;
			$t['recent']['formated'] = $t['stats']['recent_items']['formated'] ?? NULL;
		};

		// On effectue une requête SQL afin de déterminer le
		// nombre total de nouveaux fichiers dans la catégorie.
		// Mais seulement s'il n'y a pas de sous-catégories ou s'il y a plus
		// d'une page. Dans le cas contraire, on additionnera plus loin
		// le nombre de nouveaux fichiers de toutes les sous-catégories.
		$add_subcats = true;
		if (Config::$params['categories_stats']
		&& (!self::$_categories || self::$_pagesCount > 1)
		&& $r = $get_recent_items(self::$infos))
		{
			$add_subcats = false;
			$tpl(Template::$data['category'], $r);
		}

		// S'il n'y a aucune sous-catégorie, on arrête là.
		if (!self::$_categories)
		{
			return;
		}

		// On détermine le nombre de nouveaux fichiers pour chaque sous-catégorie.
		$recent = ['images' => 0, 'items' => 0, 'videos' => 0];
		foreach (self::$_categories as &$i)
		{
			// Si la catégorie ne contient pas de fichiers récents,
			// inutile d'aller plus loin.
			if (strtotime($i['cat_lastadddt']) < Item::getRecentTimestamp())
			{
				continue;
			}

			if ($r = $get_recent_items($i))
			{
				if ($add_subcats)
				{
					$recent['images'] += $r['images'];
					$recent['items'] += $r['items'];
					$recent['videos'] += $r['videos'];
				}
				$tpl(Template::$data['categories'][$i['cat_id']], $r);
			}
			else
			{
				return;
			}
		}

		// Nombre de nouveaux fichiers pour la catégorie courante.
		if (Config::$params['categories_stats'] && $add_subcats)
		{
			$tpl(Template::$data['category'], $recent);
		}
	}

	/**
	 * Retourne les tags de la catégorie.
	 *
	 * @param array $cat_infos
	 *   Informations de la catégorie.
	 * @param int $limit
	 *   Nombre de tags à récupérer.
	 *
	 * @return array
	 */
	public static function getTags(array &$cat_infos, int $limit = 15): array
	{
		// Récupération des informations.
		$sql_limit = $limit ? "LIMIT 0,$limit" : "";
		$sql = "SELECT t.*,
					   COUNT(*) AS tag_items_count
				  FROM {tags} AS t,
					   {tags_items} AS ti,
					   {items} AS i,
					   {categories} AS cat
				 WHERE %s
				   AND %s
				   AND t.tag_id = ti.tag_id
				   AND ti.item_id = i.item_id
				   AND i.cat_id = cat.cat_id
				   AND i.item_path LIKE ?
				   AND item_status = '1'
			  GROUP BY t.tag_id
			  ORDER BY tag_items_count DESC, t.tag_id ASC
					   $sql_limit";
		$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
		$path = $cat_infos['cat_id'] > 1
			? DB::likeEscape($cat_infos['cat_path']) . '/%'
			: '%';
		if (!DB::execute($sql, $path) || !$tags = DB::fetchAll('tag_name'))
		{
			return [];
		}

		// Paramètres pour déterminer le poids des tags.
		$big = current($tags)['tag_items_count'];
		$small = end($tags)['tag_items_count'];
		$diff = $big - $small;
		$increment = $diff === 0 ? 1 : $diff / 9;

		// On tri les tags par ordre alphabétique.
		uksort($tags, 'Utility::alphaSort');

		// Formatage des informations.
		if (!$cat_infos['cat_a_videos'] && $cat_infos['cat_a_images'])
		{
			$items = __('%s photos');
			$item = __('%s photo');
		}
		if ($cat_infos['cat_a_videos'] && !$cat_infos['cat_a_images'])
		{
			$items = __('%s vidéo');
			$item = __('%s vidéo');
		}
		if ($cat_infos['cat_a_videos'] && $cat_infos['cat_a_images'])
		{
			$items = __('%s photos ou vidéos');
			$item = __('%s photo ou vidéo');
		}
		$category_link = $cat_infos['cat_type'] . '/' . $cat_infos['cat_id'] . '/tag/';
		$tags_formated = [];
		foreach (array_values($tags) as &$i)
		{
			$tags_formated[] =
			[
				'count' => $i['tag_items_count'],
				'link' => App::getURL($category_link . $i['tag_id'] . '-' . $i['tag_url']),
				'name' => $i['tag_name'],
				'text' => $i['tag_items_count'] > 1
					? sprintf($items, $i['tag_items_count'])
					: sprintf($item, $i['tag_items_count']),
				'weight' => intval(
					($i['tag_items_count'] - $small) / $increment
				) + 1
			];
		}

		return $tags_formated;
	}

	/**
	 * Page de mot de passe.
	 *
	 * @return int
	 */
	public static function password(): int
	{
		Template::set('password', ['error' => 0]);

		$r = self::checkPassword($_GET['section_2'], (int) $_GET['value_1'],
			$_POST['password'] ?? '', isset($_POST['remember']));

		if ($r == 'db_error')
		{
			return -1;
		}
		if ($r == 'not_found')
		{
			return 0;
		}
		if ($r == 'bruteforce')
		{
			die('Brute force attack detected.');
		}
		if (substr($r, 0, 9) == 'redirect:')
		{
			App::redirect(substr($r, 9));
		}
		if ($r == 'password_error')
		{
			Template::set('password', ['error' => 1]);
		}

		return 1;
	}
}
?>