<?php
declare(strict_types = 1);

require_once(__DIR__ . '/Gallery.class.php');

/**
 * Gestion des pages.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class GalleryPages extends Gallery
{
	/**
	 * Récupération des appareils photos.
	 *
	 * @return int
	 */
	public static function cameras(): int
	{
		Template::set('cameras', []);

		// Récupération des informations de la catégorie.
		$cat_infos = self::_getCategoryInfos(
			$_GET['cameras_id'] ?? 1, __('Appareils photos de %s')
		);
		if (!is_array($cat_infos))
		{
			return $cat_infos;
		}

		// Récupération des appareils photos.
		$path = $cat_infos['cat_id'] > 1
			? DB::likeEscape($cat_infos['cat_path']) . '/%'
			: '%';
		$sql = "SELECT cam_m.camera_model_id,
					   camera_model_name,
					   camera_model_url,
					   cam_b.camera_brand_id,
					   camera_brand_name,
					   camera_brand_url,
					   COUNT(*) AS count
				  FROM {cameras_models} AS cam_m
			 LEFT JOIN {cameras_brands} AS cam_b USING (camera_brand_id)
		     LEFT JOIN {cameras_items} AS cam_i USING (camera_model_id)
		     LEFT JOIN {items} AS i USING (item_id)
		     LEFT JOIN {categories} AS cat USING (cat_id)
				 WHERE item_status = '1'
				   AND %s
				   AND %s
				   AND i.item_path LIKE ?
			  GROUP BY cam_m.camera_model_id, cam_b.camera_brand_id
			  ORDER BY camera_brand_name ASC, camera_model_name ASC";
		$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
		if (!DB::execute($sql, $path))
		{
			return -1;
		}
		$cameras = DB::fetchAll();
		$cameras_formated = [];
		foreach ($cameras as &$i)
		{
			$cameras_formated[] =
			[
				'count' => (int) $i['count'],
				'brand' => $i['camera_brand_name'],
				'brand_link' => App::getURL($cat_infos['cat_type'] . '/' . $cat_infos['cat_id']
					. '/camera-brand/' . $i['camera_brand_id'] . '-' . $i['camera_brand_url']),
				'model' => $i['camera_model_name'],
				'model_link' => App::getURL($cat_infos['cat_type'] . '/' . $cat_infos['cat_id']
					. '/camera-model/' . $i['camera_model_id'] . '-' . $i['camera_model_url'])
			];
		}
		Template::set('cameras', $cameras_formated);

		return 1;
	}

	/**
	 * Gestion de la page contact.
	 *
	 * @return int
	 */
	public static function contact(): int
	{
		// Message de confirmation.
		if ($_GET['q'] == 'contact/confirm')
		{
			Report::success(__('Votre message a été envoyé.'));
			return 1;
		}

		// Paramètres de template.
		$contact = Config::$params['pages_params']['contact'];
		$maxlength =
		[
			'name' => 64,
			'email' => 255,
			'subject' => 70,
			'message' => 5000
		];
		Template::set('contact',
		[
			'maxlength' => $maxlength,
			'message' => Utility::isEmpty($contact['message']) ? NULL : $contact['message']
		]);
		Template::set('field_error', '');

		// On vérifie que la page est activée.
		if (!$contact['status'])
		{
			return 0;
		}

		// Si le formulaire n'a pas été envoyé, on arrête là.
		if (empty($_POST['send']))
		{
			return 1;
		}

		// Vérification par un faux champ de formulaire.
		if (!isset($_POST['f_email']) || $_POST['f_email'] !== '')
		{
			Report::warning('Bot detected.');
			return 1;
		}

		// Vérification de chaque champ.
		foreach ($maxlength as $field => $maxlength)
		{
			// On vérifie que le champ a été rempli.
			if (!isset($_POST[$field]) || Utility::isEmpty($_POST[$field]))
			{
				Report::warning(__('Certains champs n\'ont pas été renseignés.'));
				Template::set('field_error', $field);
				return 1;
			}

			// On vérifie la longueur du champ.
			$_POST[$field] = Utility::trimAll($_POST[$field]);
			if (mb_strlen($_POST[$field]) > $maxlength)
			{
				if ($field == 'message')
				{
					Report::warning(__('Votre message ne doit pas dépasser %s caractères.'));
					Template::set('field_error', $field);
				}
				return 1;
			}
		}

		// Vérification du format de l'adresse de courriel.
		if (($r = User::checkEmailFormat($_POST['email'])) !== TRUE)
		{
			Report::warning($r);
			Template::set('field_error', 'email');
			return 1;
		}

		// Vérification par listes noires.
		$r = App::blacklists('contact', $_POST['name'], $_POST['email'], $_POST['message'],
		[
			'name' => $_POST['name'],
			'email' => $_POST['email'],
			'subject' => $_POST['subject'],
			'message' => $_POST['message']
		]);
		if (is_array($r))
		{
			$field = '';
			if ($r['list'] == 'emails')
			{
				$field = 'email';
			}
			if ($r['list'] == 'names')
			{
				$field = 'name';
			}
			if ($r['list'] == 'words')
			{
				$field = 'message';
			}
			Report::warning($r['text']);
			Template::set('field_error', $field);
			return 1;
		}

		// Message du courriel.
		$gallery_url = GALLERY_HOST . App::getURLGallery();
		$message = $_POST['message'];
		$message .= "\n\n--\n";
		$message .= sprintf(__('Ce message vous a été envoyé par '
			. 'le formulaire de contact de la galerie %s.'), $gallery_url) . "\n";
		$message .= sprintf(__('IP expéditeur : %s'), $_SERVER['REMOTE_ADDR']) . "\n";

		// Envoi du courriel.
		$mail = new Mail();
		$mail->messages[] =
		[
			'to' => $contact['email'],
			'from' => $_POST['email'],
			'name' => $_POST['name'],
			'subject' => $_POST['subject'],
			'message' => $message
		];
		if (!$mail->send())
		{
			return -1;
		}

		// Log d'activité.
		App::logActivity('contact', '',
		[
			'from' => $_POST['email'],
			'name' => $_POST['name'],
			'subject' => $_POST['subject'],
			'message' => $_POST['message']
		]);

		// Confirmation de l'envoi du message.
		App::redirect('contact/confirm');
		return 1;
	}

	/**
	 * Pages personnalisées.
	 *
	 * @return int
	 */
	public static function custom(): int
	{
		if (!isset(Config::$params['pages_params']['custom_' . $_GET['page_id']]))
		{
			return 0;
		}

		$params = Config::$params['pages_params']['custom_' . $_GET['page_id']];
		if (!$params['status'])
		{
			return 0;
		}

		$content = NULL;
		$filename = NULL;
		if (isset($params['filename'])
		&& preg_match('`^[-a-z0-9_]{1,64}\.php$`i', (string) $params['filename']))
		{
			$filename = GALLERY_ROOT . '/files/pages/' . $params['filename'];
		}
		else
		{
			$content = $params['content'];
		}

		Template::set('custom_page',
		[
			'title' => $params['title'],
			'content' => $content,
			'filename' => $filename
		]);

		return 1;
	}

	/**
	 * Récupération des tags.
	 *
	 * @return int
	 */
	public static function tags(): int
	{
		Template::set('tags', []);

		// Récupération des informations de la catégorie.
		$cat_infos = self::_getCategoryInfos($_GET['tags_id'] ?? 1, __('Tags de %s'));
		if (!is_array($cat_infos))
		{
			return $cat_infos;
		}

		// Récupération des tags.
		require_once(__DIR__ . '/GalleryCategory.class.php');
		$tags = GalleryCategory::getTags($cat_infos, 0);
		Template::set('tags', $tags);

		return 1;
	}

	/**
	 * Récupération des informations des fichiers
	 * et des catégories à géolocaliser.
	 *
	 * @return int
	 */
	public static function worldmap(): int
	{
		$worldmap = Config::$params['pages_params']['worldmap'];
		Template::set('worldmap', [
			'latitude' => $worldmap['center_lat'],
			'longitude' => $worldmap['center_long'],
			'zoom' => $worldmap['zoom']
		]);
		$coords = [];

		// Récupération des fichiers à géolocaliser.
		$sql = "SELECT item_id,
					   item_type,
					   item_path,
					   item_url,
					   item_desc,
					   item_duration,
					   item_lat,
					   item_long,
					   item_name,
					   item_adddt,
					   cat_url
				  FROM {items} AS i
			 LEFT JOIN {categories} AS cat USING (cat_id)
				 WHERE item_status = '1'
				   AND %s
				   AND %s
				   AND item_lat IS NOT NULL
				   AND item_long IS NOT NULL";
		$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
		if (!DB::execute($sql))
		{
			return -1;
		}
		if ($items = DB::fetchAll())
		{
			foreach ($items as &$i)
			{
				if (!isset($coords[$i['item_lat'] . ' ' . $i['item_long']]))
				{
					$coords[$i['item_lat'] . ' ' . $i['item_long']] =
					[
						'categories' => [],
						'items' => [],
						'lat' => $i['item_lat'],
						'long' => $i['item_long']
					];
				}
				$coords[$i['item_lat'] . ' ' . $i['item_long']]['items'][] =
				[
					'duration' => $i['item_duration']
						? App::formatDuration($i['item_duration'])
						: NULL,
					'id' => $i['item_id'],
					'link' => App::getURL('item/' . $i['item_id'] . '-' . $i['item_url']),
					'thumb_src' => function($size = '', $quality = '') use ($i)
					{
						return htmlspecialchars(
							App::getThumbSource('item', $i, (string) $size, (string) $quality)
						);
					},
					'title' => $i['item_name'],
					'type' => Item::isImage($i['item_type']) ? 'image' : 'video'
				];
			}
		}

		// Récupération des catégories à géolocaliser.
		$sql = "SELECT cat.cat_id,
					   thumb_id,
					   cat_path,
					   cat_name,
					   cat_desc,
					   cat_url,
					   cat_lat,
					   cat_long,
					   cat_crtdt,
					   cat_filemtime,
					   CASE WHEN cat_filemtime IS NULL
						    THEN 'category' ELSE 'album'
							 END AS cat_type,
					   item_id,
					   item_path,
					   item_type,
					   item_width,
					   item_height,
					   item_adddt
				  FROM {categories} AS cat
			 LEFT JOIN {items} AS i
					ON cat.thumb_id = i.item_id
				 WHERE cat_status = '1'
				   AND %s
				   AND %s
				   AND cat_lat IS NOT NULL
				   AND cat_long IS NOT NULL";
		$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
		if (!DB::execute($sql))
		{
			return -1;
		}
		if ($categories = DB::fetchAll())
		{
			foreach ($categories as &$i)
			{
				if (!isset($coords[$i['cat_lat'] . ' ' . $i['cat_long']]))
				{
					$coords[$i['cat_lat'] . ' ' . $i['cat_long']] =
					[
						'categories' => [],
						'items' => [],
						'lat' => $i['cat_lat'],
						'long' => $i['cat_long']
					];
				}
				$coords[$i['cat_lat'] . ' ' . $i['cat_long']]['categories'][] =
				[
					'id' => $i['item_id'],
					'link' => App::getURL(
						$i['cat_type'] . '/' . $i['cat_id'] . '-' . $i['cat_url']
					),
					'thumb_src' => function($size = '', $quality = '') use ($i)
					{
						return htmlspecialchars(
							App::getThumbSource('cat', $i, (string) $size, (string) $quality)
						);
					},
					'title' => $i['cat_name']
				];
			}
		}

		Template::set('coords', array_values($coords));

		return 1;
	}



	/**
	 * Récupération des informations de la catégorie.
	 *
	 * @param int $cat_id
	 * @param string $breadcrumb_text
	 *
	 * @return mixed
	 */
	private static function _getCategoryInfos(int $cat_id, string $breadcrumb_text)
	{
		$sql = "SELECT cat.*,
					   %s,
					   CASE WHEN cat_filemtime IS NULL
						    THEN 'category' ELSE 'album'
							 END AS cat_type
				  FROM {categories} AS cat
				 WHERE %s
				   AND cat.cat_id = ?
				   AND cat_status = '1'";
		$sql = sprintf($sql, SQL::catPassword('select'), SQL::catPerms());
		if (!DB::execute($sql, $cat_id))
		{
			return -1;
		}
		if (!$cat_infos = DB::fetchRow())
		{
			return 0;
		}

		// L'utilisateur a-t-il entré le bon mot de passe ?
		if (!self::checkCookiePassword($cat_infos))
		{
			return 0;
		}

		// Réduction des stats.
		if (!self::changeCatStats($cat_infos))
		{
			return -1;
		}

		// Fil d'Ariane.
		$breadcrumb =
		[
			'category_link' => App::getURL(),
			'category_name' => __('la galerie'),
			'category_text' => '%s',
			'text' => $breadcrumb_text
		];
		if ($cat_infos['cat_id'] > 1)
		{
			$breadcrumb['category_link'] = App::getURL(
				$cat_infos['cat_type'] . '/' . $cat_infos['cat_id'] . '-' . $cat_infos['cat_url']
			);
			$breadcrumb['category_name'] = $cat_infos['cat_name'];
			$breadcrumb['category_text'] = $cat_infos['cat_type'] == 'category'
				? __('la catégorie %s')
				: __('l\'album %s');
		}
		Template::set('breadcrumb', $breadcrumb);

		return $cat_infos;
	}
}
?>